/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the interface to the InSoft error (actually, debug message)
 *   reporting system.
 *
 * Requires:
 *  None
 *
 * Includes:
 *  None
 *
 * Implemented by:
 *  ISerror.c
 *
 * Types:
 *  typedef enum {...} ISerror_init_Result
 *
 * Functions:
 *  const char* ISerror_progname(void);
 *  int ISdebugMsg(const char* format, ...)
 *  int ISwarnMsg(const char* format, ...)
 *  int ISdiagMsg(const char* format, ...)
 *  ISerror_init_Result ISerror_init(const char* progname);
 *
 * Author:
 *  francis, 1 Feb. 1994
 *
 * Notes:
 *  The error system checks the environment variable INSOFT_ERROR_PRINTER;
 *   if it is not set, no error messages will be printed.  If it is set to
 *   "", error messages will be printed to stderr; otherwise, it is
 *   interpreted as a pathname to which to print messages.
 *
 * Change Log:
 *  $Log$
 */

#ifndef _INSOFT_ERROR_HANDLER_H_
#define _INSOFT_ERROR_HANDLER_H_

#ifdef __cplusplus
extern "C" {
#endif

/*
 * const char* ISerror_progname(void);
 *
 * Description:
 *  Returns a pointer to the program name used by ISerror().  For use in
 *   functions that need to print the program name; e.g., usage().
 *
 * Preconditions:
 *  None.
 *
 * Postconditions:
 *  None.
 *
 * Externals:
 *  Uses the internal variable (static in the module ISerror.c)
 *   which stores the program name.
 */

const char* ISerror_progname(void);

/*
 * int IS{debug,warn,diag}Msg(const char* format, ...)
 *
 * Description:
 *  Wrappers around vfprintf(3); format is interpreted as for any *printf()
 *   function.  Prints the specified error message to the error file
 *   specified by INSOFT_ERROR_PRINTER (see Notes: in header header), or
 *   not at all if INSOFT_ERROR_PRINTER is null.
 *  The exception is ISerrorMsg(), which will always go to stderr.
 *
 * Return value:
 *  Returns -1 on error, or the number of items printed otherwise.
 *   If printing is disabled, returns 0.
 *
 * Preconditions:
 *  The InSoft error system must have been initialized with
 *   ISerror_init().  If not, printing is effectively disabled.
 *
 * Externals:
 *  Uses module-static variables for program name and error file.
 */
int ISdiagMsg(const char* format, ...);
int ISwarnMsg(const char* format, ...);
int ISdebugMsg(const char* format, ...);
int ISerrorMsg(const char* format, ...);

/* Result codes for ISerror_init(), below.  Maybe be ORed together.
 */
typedef
enum {ISerror_init_OK=0, ISerror_init_cantdup=1, ISerror_init_badfile=2}
ISerror_init_Result;

/*
 * ISerror_init_Result ISerror_init(const char* progname);
 *
 * Description:
 *  Initializes the internal cache of the program's name and the error
 *   file (or none).
 *
 * progname is strdup()ed.  If strdup() fails, the cached program name
 *  is not changed (the initial value is "<progname>"); this means that a
 *  process can set INSOFT_ERROR_PRINTER and call ISerror_init(NULL) later,
 *  changing the file but not the program name.
 *
 * Return value:
 *  Returns the OR of flags representing problems encountered:
 *   ISerror_init_cantdup: could not strdup() the program name
 *   ISerror_init_badfile: could not open the printer file.
 *
 * Preconditions:
 *  None.
 *
 * Postconditions:
 *  The InSoft error system is initialized and ready to use.  Even if
 *   a problem is encountered, the system will function; it may not
 *   function perfectly, is all.
 *
 * Externals:
 *  Uses (initializes) module-static variables for program name and
 *   error file.
 *
 */
ISerror_init_Result ISerror_init(const char* progname);

ISerror_init_Result ISerror_setDiag(const char* controlVar);
ISerror_init_Result ISerror_setWarn(const char* controlVar);
ISerror_init_Result ISerror_setDebug(const char* controlVar);
ISerror_init_Result ISerror_setError(const char* controlVar);

int ISerror_flushDiag(void);
int ISerror_flushWarn(void);
int ISerror_flushDebug(void);
int ISerror_flushError(void);

#ifdef __cplusplus
}
#endif

#endif /*_INSOFT_ERROR_HANDLER_H_*/
