#ifndef _DVE_AV_TONES_H_
#define _DVE_AV_TONES_H_

#ifdef __cplusplus
extern "C" {
#endif
  
#include <dve/AVCtl/core.h>
  

/* The tones/file-player API generates DTMF tones and plays audio files.
 * At any given time, at most one DTMF tone and one audio file playback 
 * may be active. Each API invocation to play a tone/file has three
 * major parameters : 
 * (1) The Play Mode : This determines whether the tone is played once or 
 *                     repeatedly.
 * (2) The Transmit Mode : This determines whether the tone should be
 *                         transmitted to the network, or played back locally,
 *                         or both.
 * (3) The Mix Mode : This determines whether the transmitted/played back
 *                    tone (determined by the transmit mode) is mixed with 
 *                    the audio stream(s) along that path.
 */

/* Determines whether file/tone should be played once or repeatedly */
typedef enum {	
  dveAVPlayModeInvalid=0,
  dveAVPlayModeOnce,
  dveAVPlayModeRepeated
  } DveAVPlayMode;

/* Determines whether file/tone should be sent to the network, or played back, 
 * or both.
 */
typedef enum {
  dveAVTransmitModeInvalid=0,
  dveAVTransmitModeNetwork,
  dveAVTransmitModeLocal,
  dveAVTransmitModeAll
  } DveAVTransmitMode;


/* Determines whether file/tone should be mixed with local/outgoing audio. 
 * Note that whether the file/tone should be transmitted or played back
 * or both is determined by DveAVTransmitMode; DveAVMixMode determines whether
 * the transmitted/played back file/tone should be mixed with incoming
 * or outgoing audio. For example, if the transmit mode of a tone/file is
 * dveAVTransmitModeAll and the mix mode is dveAVMixModeNetwork, then the
 * tone/file is mixed with outgoing audio but not with incoming audio. This
 * means that when the tone is played back locally, it pre-empts all incoming
 * audio streams for the duration of the playback. 
 */
typedef enum {
  dveAVMixModeInvalid=0,
  dveAVMixModeNone,      /* Do not mix on any path */
  dveAVMixModeLocal,     /* Mix with incoming audio, but not on the network */
  dveAVMixModeNetwork,   /* Mix with outgoin audio, but not with incoming */
  dveAVMixModeAll        /* Mix with both incoming and outgoing audio */
  } DveAVMixMode;

/* The following is a description of various gains from tone generators to
 * either the network or local playback. In the description below, 
 * F - File player (tone generator)
 * D - DTMF tone generator
 * N - network
 * M - Currently active input device
 * S - Currently active output device(s)
 */
typedef enum {
  dveAVGainTypeInvalid=0,
  dveAVGainTypeFToN,   /* Tone generator to network */
  dveAVGainTypeMToN,   /* Input device to network, also audio sending vol */
  dveAVGainTypeDToN,   /* DTMF tone generator to network */
  dveAVGainTypeFToS,   /* Tone generator to output device */
  dveAVGainTypeDToS,   /* DTMF tone generator to output device */
  dveAVGainTypeNToS    /* Network to output device - audio play volume */
  } DveAVGainType;


/*
 * dveAVStartDTMFTone
 *    Start playing a DTMF tone.
 *
 * Input : session : Session returned by dveAVOpenSession()
 *         dtmfString : NULL terminated string specifying the digits in tone
 *         duration : Duration of tone (?)
 *         interDigitTime : Inter-digit time in milliseconds
 *         mode : Playback only, or transfer to network, or both?
 *         mix : Determines the paths along which the tone is mixed
 *
 * Output :
 * Return :
 * Notes :
 */
void dveAVStartDTMFTone(DveAVSession session, char *dtmfString, 
			u_int32 duration, u_int32 interDigitTime,
			DveAVPlayMode pmode,
			DveAVTransmitMode tmode, DveAVMixMode mixMode);

/*
 * dveAVStopDTMFTone
 *   Stop a currently playing DTMF tone, if any
 * 
 * Input : session : Session returned by dveAVOpenSession()
 * Output :
 * Return :
 * Notes :
 */
void dveAVStopDTMFTone(DveAVSession session);


/*
 * dveAVSetFileGain
 *   Set a gain value in a path from a tone generator to output dev/network or
 *   from network to output device.
 * 
 * Input : session : Session returned by dveAVOpenSession()
 *         gtype : Gain tag
 *         value : Value of gain on a scale from 0 to 100
 * Output :
 * Return :
 * Notes :
 */
void dveAVSetFileGain(DveAVSession session, DveAVGainType gtype, int value);

/*
 * dveAVGetFileGain
 *   Get the value of a gain from  a tone generator to output dev/network or
 *   from network to output device.
 *
 * Input :
 * Output :
 * Return : The value of specified gain, -1 on failure
 * Notes :
 */
int dveAVGetFileGain(DveAVSession session, DveAVGainType gtype);

/*
 * dveAVPlayFile
 *    Replaces the original dveAVPlayTone() routine. A tone is now just an
 *    audio file - the programmer is responsible for its location.
 * 
 * Input : domain : Currently, only dveAVDomainAudio will be supported
 *         file : Pathname of file
 *         format : Reserved; for now, pass in NULL
 *         sleepTime : Time between successive playbacks of the file
 *         tmode : Transmit to network, playback, or both?
 *         pmode : Play once, or repeatedly?
 *         mmode : Determines the path along which the tone is mixed
 * Output :
 * Return :
 * Notes : Only one audio file can be played at any time.
 */
void dveAVPlayFile(DveAVSession session, DveAVDomain domain,
		   char *file, char *format,
		   int sleepTime, 
		   DveAVTransmitMode tmode,
		   DveAVPlayMode pmode,
		   DveAVMixMode mmode);
/*
 * dveAVStopFilePlayback
 *    Stop playing an audio file that is currently playing, if any. 
 *
 * Input : 
 * Output :
 * Return :
 * Notes :
 */
void dveAVStopFilePlayback(DveAVSession session);

#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_TONES_H_ */
