#ifndef _PROFILE_H_
#define _PROFILE_H_

#ifdef __cplusplus
extern "C" {
#endif

/* Unless otherwise noted, the return value for a function returning
 *  int is 0 on success or <0 on error.  For a function returning a
 *  pointer type (e.g., pluginObject), a NULL return indicates an error.
 *  On error, consult dvePluginMgrErrno (see below) for the error.
 */

#include <sys/param.h>
#include <ISprofile.h>

#define         PLUGIN_UNMAPPED	0
#define         PLUGIN_MAPPED	1
#define         MAXPLUGINS      256

/*
 * Needed Types for Plugin Objects...
 */
typedef struct _pluginObjectType        *pluginObject;
 
struct _pluginObjectType {
        const char	*name;
        const char	*title;
        int         	processID;
        u_int32         pluginReady;
        DvePort         port;
        pluginObject    next;
        pluginObject    prev;
        u_int32         autoload;
        u_int32         createButton;
        u_int32         videoFrameCount;
        u_int32         averageBandwidth;
        u_int32         peakBandwidth;
	ISprofile	profile;
        char            basePath[MAXPATHLEN];
};

/* 
 * Callback invoked when the Plugin exits/Gets Ready... 
 */
typedef void (*PluginMgrCallbackProc)(pluginObject plug);

/*
 * Error conditions that could occur and are relayed back to the
 * user application via the dvePluginMgrErrno variable.
 */
typedef enum {
	dvePluginMgrUnknownError	= 0,
	dvePluginMgrLowMemory,
	dvePluginMgrInvalidPluginObject,
	dvePluginMgrInvalidSessionHandle,
	dvePluginMgrInvalidPluginPort,
	dvePluginMgrSendDataPacketError,
	dvePluginMgrInvalidInitFile,
	dvePluginMgrLibraryNotInitialized,
	dvePluginMgrErrorOnFork
} dvePluginMgrErrorTypes;

/* 
 * External variables for the user...
 */
extern u_int32		dvePluginMgrErrno;
extern pluginObject     dvePluginList[MAXPLUGINS];

/*
 * All API calls...
 *
 *****
 ***** Library initilization and cleanup routines
 *****
 */

/* 
 * Initializing the library
 *
 *
 */

int
dvePluginMgrInitialize(DveSession dveSession, char *initFile,
		       PluginMgrCallbackProc readyCallback,
		       PluginMgrCallbackProc deathCallback);

/* 
 * Locating all available Plugins
 * 
 * This function will have to scan through the predefined search paths
 * and initilization files and located the Plugins that have been
 * defined for that user. There will need to be some internal
 * representation that will be exposed to the user in some manner as
 * well as several convience functions that will be available to the
 * developer for accessing the Plugin objects.
 */
int
dvePluginMgrIdentifyAvailablePlugins(DveSession dveSession);

/* 
 * Close the Library
 * 
 * This function needs to be done as a cleanup routine for the
 * library.  This will deallocate any and all allocated memory as well
 * as cleanup any possible OpenDVE connections.
 */
void
dvePluginMgrClose(void);

/*
 *****
 ***** Routines for starting and stopping Plugins.
 *****
 */

/*
 * Start a Plugin
 * 
 * Once all of the Plugins available to the user have been defined,
 * There will need to be a mechanism to start this process. One
 * primary feature of starting an OpenDVE Plugin is the state in which
 * the Plugin will show itself to the end user. These two states are
 * Mapped and Unmapped. (i.e. Display yourself on the desktop or start
 * in the background and do not display)
 */
int
dvePluginMgrStartPlugin(DveSession dveSession, pluginObject plug, 
						u_int32 mapState);
/*
 * Terminate a Plugin
 * 
 * Once a Plugin has been started, there will be a mechanism needed to
 * tell it to terminate and exit. This will send the OpenDVE message
 * to this process to have it exit gracefully.
 */ 
int
dvePluginMgrQuitPlugin(DveSession dveSession, pluginObject plug);

/*
 * Start all Plugins
 * 
 * There will be times when you will need to start all available
 * Plugins that have been configured for this user. In this case, it
 * would be useful to have a single call to start all of these
 * processes in the background. It will also be important to have the
 * ability to control the mapping of this tool on the users desktop.
 * (i.e. Display yourself on the desktop or start in the background
 * and do not display)
 */
int
dvePluginMgrStartAllPlugins(DveSession dveSession, u_int32 mapState);

/*
 * Stop all Plugins
 * As it would be useful to start all Plugins available to the user,
 * it would also be very important to have the ability to stop all the
 * Plugins that are running on the users desktop.
 */
int
dvePluginMgrStopAllPlugins(DveSession dveSession);

/*
 *****
 ***** Routines for controlling the mapped state of the Plugins that are
 ***** currently running.
 *****
 */

/*
 * Tell a Running Plugin to Show Itself
 * There are many times when a Plugin is started and then placed into
 * an unmapped state. Then when a user requests for this Plugin to be
 * shown to the user, a message needs to be sent to that Plugin to
 * have it mapped to the user.
 */
int
dvePluginMgrShowPlugin(DveSession dveSession, pluginObject plug);

/*
 *
 */
int 
dvePluginMgrShowAllPlugins(DveSession dveSession)



/*
 * Tell a Running Plugin to Hide Itself
 *
 * There are times when a running Plugin may need to be told to Hide
 * itself from the user by unmapping its interface. A call needs to be
 * available to have access to this type of function.
 */
int
dvePluginMgrHidePlugin(DveSession dveSession, pluginObject plug);

/*
 *
 */
int 
dvePluginMgrHideAllPlugins(DveSession dveSession)


/*
 *****
 ***** General informational routines for the Plugins.
 *****
 */


/*
 *****
 ***** Routines for locating a Plugin object by different keys.
 *****
 */
/*
 * Locate Plugin Object by Title
 * 
 * Plugins will need to be able to be located by various tags. One
 * item that identifies a Plugin is the Plugin title. This title it
 * typically the user level name for the component.
 */
pluginObject
dvePluginMgrLocatePluginByTitle(char *tag);

/*
 * Locate Plugin Object by Name
 * Plugins will need to be able to be located by various tags. One
 * item that identifies a Plugin is the Plugin Name. This is the name
 * of the executable as defined by the Plugin developer.
 */
pluginObject
dvePluginMgrLocatePluginByName(char *tag);

/*
 * Locate Plugin Object by Process ID
 * 
 * Plugins will need to be able to be located by various information.
 * One item that identifies a Plugin once it has been started is the
 * Plugin's process id.
 */
pluginObject
dvePluginMgrLocatePluginByProcessID(u_int32 pid);

/*
 * Locate Plugin Object by Port Number
 * 
 * Plugins will need to be able to be located by various information.
 * The most important item that is guaranteed to be unique for a
 * Plugin that is available for a developer is the Plugin's Port
 * Number. This is a numeric value that is used when connecting to the
 * Conference Engine.
 */
pluginObject
dvePluginMgrLocatePluginByPortNumber(DvePort port);

#ifdef __cplusplus
}
#endif

#endif
