/*
 *
 * Name:        OpenDVE_procs.c
 *
 * Description: Provide simple OpenDVE functions to ISCameraToolPluginMotif.
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <sys/types.h>

#ifdef XVIEW
#include <xview/xview.h>
#endif

#ifdef XT
#include <X11/Intrinsic.h>
#endif

/*
 * OpenDVE related include files.
 */
#include <dve/dve.h> 
#include <dve/pluginManagement.h> 

/*
 * External callback functions.
 */
extern void starting(DveSession dveSession, DveEvent* event);
extern void MemberLeaving(DveSession dveSession, DveEvent* event);
extern void NewMemberAdded(DveSession dveSession, DveEvent* event);

/*
 * Local callback funtions.
 */
static void EngineShutDown(DveSession dveSession, DveEvent* event);
static void PluginControlMsg(DveSession dveSession, 
                             DveUserHandle sender,
                             DveDataPacket* packet);
static void PluginMessageCallback(DveSession dveSession,
                                  DveUserHandle sender,
                                  DveDataPacket* packet);

/*
 * Needed variables.
 */
DveSession dveSession = NULL;

/*
 * This is here for the initialization of OpenDVE.
 */
#ifdef RAW
   DveSession 
   InitializeOpenDVE(DvePort pluginPort)
#endif

#ifdef XVIEW
   DveSession 
   InitializeOpenDVE(Xv_opaque win, DvePort pluginPort)
#endif

#ifdef XT
   DveSession 
   InitializeOpenDVE(XtAppContext app_context, DvePort pluginPort)
#endif
{
   /*
    * Open the DVE Session.
    */
   if (!(dveSession=dveOpenSession(pluginPort)))
   {
      return(NULL);
   }

   /*
    * Set the callbacks.  
    */
   dveSetEventCallback(dveSession, dveEventTagStarting, starting);
   dveSetEventCallback(dveSession, dveEventTagEngineShutdown, EngineShutDown);

   dveSetUserCallback(dveSession,dvePluginManagementProtocol, PluginControlMsg);
   dveSetEventCallback(dveSession,dveEventTagMemberLeft,MemberLeaving);
   dveSetEventCallback(dveSession,dveEventTagMemberAccepted,NewMemberAdded);
   dveSetUserCallback(dveSession, pluginPort, PluginMessageCallback);

   /*
    * Receive the first event.
    */
   dveReceiveEvent(dveSession, dveEventTagStarting);

#ifdef XVIEW
   /*
    * Register with XView.
    */
   if (dveRegisterWithXView(dveSession, win)<0)
   {
      dveCloseSession(dveSession);
      return(NULL);
   }
#endif

#ifdef XT
   /*
    * Register with Xt.
    */
   if (dveRegisterWithXt(dveSession, app_context)<0)
   {
      dveCloseSession(dveSession);
      return(NULL);
   }
#endif

   dveReportPresence(dveSession);

   return(dveSession);
}

/*
 * Orderly shutdown of the OpenDVE session.
 */
void
QuitPlugin(DveSession dveSession)
{
   dveCloseSession(dveSession);

#ifdef XT
   dveUnregisterWithXt(dveSession);
#endif

#ifdef XVIEW
   dveUnregisterWithXView(dveSession);
#endif
   exit(0);
}

/*
 * QuitPlugin if requested.
 */
static void 
PluginControlMsg(DveSession dveSession, DveUserHandle sender,
                 DveDataPacket* packet)
{
   switch ((DvePluginManagementMsgTag)packet->tag)
   {
      case dvePluginManagementMsgTagQuit:
         QuitPlugin(dveSession);
         break;
      case dvePluginManagementMsgTagShow:
         ShowPlugin();
         break;
      case dvePluginManagementMsgTagHide:
         HidePlugin();
         break;
   }
}

/*
 * If we notice that the conference engine has shutdown, we should
 * also shutdown politely.
 */
static void
EngineShutDown(DveSession dveSession, DveEvent* event)
{
   QuitPlugin(dveSession);
}

/*
 *  This is for an orderly shutdown of the session.
 */
void
ClosePluginSession(DveSession dveSession)
{
#ifdef XT
   dveUnregisterWithXt(dveSession);
#endif

#ifdef XVIEW
   dveUnregisterWithXView(dveSession);
#endif
   dveCloseSession(dveSession);
}

/*
 * Callback for all user events on port number, REMOTE_CAMERA_PORT.
 */
static void 
PluginMessageCallback(DveSession dveSession, DveUserHandle sender, DveDataPacket* packet)
{
   ProcessPluginMessage(packet->tag, packet->data, sender);
}

/*
 * Passses data from this plugin to other conferenced members.
 */
int
SendPluginMessageGeneric(DveSession dveSession, u_int32 protocol, DvePort port,
			u_int32 messageType, int dest, void *data, u_int32 size)
{
   DveDataPacket *packet;
   DveConfParticipant member;
   int returnCode;

   packet = (DveDataPacket*) dveMallocDataPacket(dveSession, size);
   packet->tag = (u_int32)messageType;
   packet->pluginProtocol = protocol;
   packet->destPort = port;

   DvePartListClrAll(&(packet->destParts));

   if (dest < 0)
   {
      DvePartListSetAll(&(packet->destParts));
   }
   else
   {
      DvePartListSet(dest,&(packet->destParts));
   }

   if (size > 0)
      memcpy(packet->data, data, size);

   returnCode = dveSendDataPacket(dveSession, packet);

   dveFreeDataPacket(packet); 
   return(returnCode);
}

/* 
 * Send a plugin message to all users currently conferenced.
 */
int
SendPluginMessage(DveSession dveSession, DvePort port, u_int32 messageType,
				void *data, u_int32 size)
{
   return(SendPluginMessageGeneric(dveSession, port, port, 
                                   messageType, -1, data, size));
}

/*
 * Send a plugin message to the identified user.
 */
int
SendPluginMessageToMember(DveSession dveSession, DvePort port, 
		DveUserHandle member, u_int32 messageType, void *data, 
		u_int32 size)
{
   return(SendPluginMessageGeneric(dveSession, port, port,
                                   messageType, member, data, size));
}

/*
 * Send a message to myself.
 */
int
SendLocalPluginMessage(DveSession dveSession, DvePort port, 
			u_int32 messageType, void *data, u_int32 size)
{
   return(SendPluginMessageGeneric(dveSession, port, port, 
                                   messageType, 0, data, size));
}

/*
 * Send a message to conference management.
 */
int
SendPluginManagementMessage(DveSession dveSession, DvePort port, 
						u_int32 messageType)
{
   return(SendPluginMessageGeneric(dveSession, dvePluginManagementProtocol, port,
                                   messageType, 0, NULL, 0));
}
