/*
 *
 * Name:	OpenDVE_procs.c
 *
 * Description: Provide simple OpenDVE functions to ISLogoPlugin*.
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <sys/types.h>

#ifdef XT
#include <X11/Intrinsic.h>
#endif

/*
 * OpenDVE related include files.
 */
#include <dve/dve.h> 
#include <dve/pluginManagement.h> 
#include <dve/clipboard.h>

#include "ISLogoPluginProtocol.h"

/* Callback */
static void PluginControlMsg(DveSession dveSession,
                             DveUserHandle sender,
                             DveDataPacket* packet);

/* Local functions */
void QuitPlugin(DveSession dveSession);
void ShutdownOpenDVEClipboard(DveClipboard clipboard);
DveClipboard InitializeOpenDVEClipboard(DveSession dveSession, 
                                        char *clipboardName,
                                        DveUserHandle userHandle);

/*
 * Needed variables.
 */
DveSession dveSession = NULL;
DveClipboard clipboard = NULL;

/*
 * This is here for the initialization of OpenDVE.
 */
#ifdef RAW
   DveSession 
   InitializeOpenDVE(DvePort pluginPort)
#endif

#ifdef XVIEW
   DveSession 
   InitializeOpenDVE(Xv_opaque win, DvePort pluginPort)
#endif

#ifdef XT
   DveSession 
   InitializeOpenDVE(XtAppContext app_context, DvePort pluginPort)
#endif
{
   /*
    * Open the DVE Session.
    */
   if (!(dveSession=dveOpenSession(pluginPort)))
   {
      return(NULL);
   }

   dveSetUserCallback(dveSession,dvePluginManagementProtocol, PluginControlMsg);

#ifdef XVIEW
   /*
    * Register with XView.
    */
   if (dveRegisterWithXView(dveSession, win)<0)
   {
      dveCloseSession(dveSession);
      return(NULL);
   }
#endif

#ifdef XT
   /*
    * Register with Xt.
    */
   if (dveRegisterWithXt(dveSession, app_context)<0)
   {
      dveCloseSession(dveSession);
      return(NULL);
   }
#endif

   dveReportPresence(dveSession);

   return(dveSession);
}

/*
 * QuitPlugin if requested.
 */
static void
PluginControlMsg(DveSession dveSession, DveUserHandle sender,
                 DveDataPacket* packet)
{
   switch ((DvePluginManagementMsgTag)packet->tag)
   {
      case dvePluginManagementMsgTagQuit:
         QuitPlugin(dveSession);
         break;
      case dvePluginManagementMsgTagShow:
         ShowPlugin();
         break;
      case dvePluginManagementMsgTagHide:
         HidePlugin();
         break;
   }
}

/*
 * Orderly shutdown of the OpenDVE session.
 */
void
QuitPlugin(DveSession dveSession)
{
   ShutdownOpenDVEClipboard(clipboard);
   dveCloseSession(dveSession);
 
#ifdef XT
   dveUnregisterWithXt(dveSession);
#endif
 
#ifdef XVIEW
   dveUnregisterWithXView(dveSession);
#endif
   exit(0);
}

/*
 *****************************************************************
 *****************************************************************
 ************* OpenDVE Clipboard Related Routines. ***************
 *****************************************************************
 *****************************************************************
 *****************************************************************
 */
/*
 * Initialize the Clipboard and set global.
 */
DveClipboard
InitializeOpenDVEClipboard(DveSession dveSession, char *clipboardName,
					DveUserHandle userHandle)
{
   DveClipboard clipboard = NULL;

   if (!clipboard)
   {
      clipboard=dveClipOpen(dveSession, clipboardName, userHandle);
   }
   return(clipboard);
}

/*
 * Close the local clipboard and free all of the associated memory.
 */
void
ShutdownOpenDVEClipboard(DveClipboard clipboard)
{
   dveClipClose(clipboard);
}

/* 
 * Copy a data item to the local clipboard.
 */
int 
CopyDataToClipboard(DveClipboard clipboard, const char* data, 
					int size, char *format)
{
   DveClipItem *item = NULL;
 
   if (size == 0)
      return(-1);

   if (!clipboard)
   {
      return(-2);
   }

   /*
    * Lock the clipboard before we attempt to alter its contents.
    */
   if (dveClipLock(clipboard)<0)
   {
      return (-3);
   }

   /*
    * Allocate the clipboard item using dveMalloc().  Allocates a block
    * of memory for use using a specialized allocator e.g. if
    * you're registered via dveRegisterWithXt(), we'll use
    * XtMalloc(), which provides more error checking than malloc().
    */
   item=(DveClipItem*)dveMalloc(sizeof(DveClipItem) + size);
   if (!item)
   {
      return (-4);
   }
      
   /*
    * Place the data in DveClipItem item in prep for dveClipPut().
    */
   strcpy(item->format, format);
   memcpy(item->data, data, size);
   item->datalen=size; 		

   /*
    * Erase the clipboard before we pass the new DveClipItem.
    */
   (void)dveClipWipe(clipboard);

   /* 
    * Place the new item, unlock the clipboard and free the space
    * handed to us by dveMalloc().
    */
   if (dveClipPut(clipboard, item)<0)
   {
      if (item)
         dveFree(item);

      (void) dveClipUnlock(clipboard);
      return (-5);
   }

   (void) dveClipUnlock(clipboard);

   if (item)
      dveFree(item);

   return (0);
}

/*
 * Attempt to create a DveClipboard named dveClipStdName (defined
 * as "Clipboard").
 */
int
InitializePluginClipboard(void)
{
   clipboard = InitializeOpenDVEClipboard(dveSession, dveClipStdName, 0);
   if (!clipboard)
      return(-1);
   else
      return(0);
}
