/*
 *
 * Name:        Plugin_procs.c
 *
 * Description: Used to demonstrate OpenDVE user interface
 *              convenience functions.               
 *            
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <string.h>
#include <sys/param.h>
#include <netinet/in.h>
#include <arpa/inet.h>

#include <dve/Xm/recipientList.h>
#include <dve/pluginManagement.h>
#include "UxXt.h"

#include "rpApplicationShell.h"
#include "ISRecipientPluginMotif.h"

extern Widget recipTestAppShell;
extern Widget label1;

DveSession dveSession=NULL;

static void HidePlugin(void);
static void ShowPlugin(void);
static void SetFooter(const char *msg);

static void starting(DveSession dveSession, DveEvent* event);
static void memberListChanged(DveSession dveSession, DveEvent* event);
static void PluginControlMsg(DveSession dveSession,
                             DveUserHandle sender,
                             DveDataPacket* packet);


static ConfdStateTag confdState=confdStateTagInvalid;

/* 
 * Callback invoked when a user selects Exit from the File menu pulldown.
 */
void recipTestQuit(void)
{
   HidePlugin();
}

/*
 * DveMotifRecipientList is an OpenDVE data type which maintains a 
 * list of selected conference members in a Motif XmList.  In this example
 * two lists are utilized, listM and listB.
 * 
 * Conference participant lists are held in the OpenDVE data types
 * partListM and partListB.
 */
DveMotifRecipientList listM=NULL;
DvePartList partListM;

DveMotifRecipientList listB=NULL;
DvePartList partListB;

void recipListCallback(DveMotifRecipientList list,
		       DveMotifRecipientListCallbackStruct* cbs);

/*
 * When the starting event is received from the Conference Engine 
 * both lists are created using the OpenDVE toolkit function 
 * dveMotifRecipientListCreate().
 */
void starting(DveSession dveSession, DveEvent* event)
{
   int i=0;

   if (!listM)
   {
      /*
       * Start with zero conference participants.
       */
      DvePartListClrAll(&partListM);

      /*
       * The DveSession connection to OpenDVE Conference Engine is 
       * represented by dveSession.  recipTestList is an XmList widg
       * by interface.  partListM is the DvePartList in which to store
       * the list of members currently selected.
       *
       * False indicates that we do NOT wish to exclude ourself
       *  from the DveMotifRecipientList
       */
      listM=dveMotifRecipientListCreate(dveSession,
					recipTestList,
					&partListM,
					False);
      if (!listM)
	return;

      /* If autoSelect is set to True, members will be selected as soon as
       * they accept (when the conference starts, all members will be
       * selected).  The default is False.
       *
       * Clearly, autoSelect is False when the list is first created.  If
       * it is set to True before any X or OpenDVE events are processed
       * for the list (immediately after creation should do), all
       * conference members will be selected.
       *
       * autoSelect is ignored for browse-mode lists.
       */
      dveMotifRecipientListSetAutoSelect(listM, True);

      dveMotifRecipientListSetCallback(listM, recipListCallback);

      DvePartListClrAll(&partListB);

      listB=dveMotifRecipientListCreate(dveSession,
					recipTestSingleList,
					&partListB,
					False);
      if (!listB)
	return;

      dveMotifRecipientListSetCallback(listB,recipListCallback);
   }
   else
   {
      /*
       * The DveMotifRecipientList lists have already been created.
       * Simply process the incoming OpenDVE event to update the list
       * of conference members.
       */
      dveMotifRecipientListHandleDveEvent(listM,event);
      dveMotifRecipientListHandleDveEvent(listB,event);
   }
}

/*
 * Simply process the incoming OpenDVE event to update the
 * list of conference members.
 */
void memberListChanged(DveSession dveSession, DveEvent* event)
{
   if (listM)
      dveMotifRecipientListHandleDveEvent(listM,event);
   if (listB)
      dveMotifRecipientListHandleDveEvent(listB,event);
}

/*
 * User activity has occurred on a DveMotifRecipientList.
 * Read the dveMotifRecipientListChangeTag from
 * DveMotifRecipientListCallbackStruct.
 */
void recipListCallback(DveMotifRecipientList list,
                       DveMotifRecipientListCallbackStruct* cbs)
{
   char tmp[50];

   memset(tmp, '\0', strlen(tmp));

   switch (cbs->tag)
   {
      case dveMotifRecipientListChangeTagAdded:
         sprintf(tmp, "Added member #%d\n",cbs->arg.added.member);
         break;
 
      case dveMotifRecipientListChangeTagDeleted:
         sprintf(tmp, "Deleted member #%d\n",cbs->arg.deleted.member);
         break;
 
      case dveMotifRecipientListChangeTagChanged:
         sprintf(tmp, "Changed only member #%d\n",cbs->arg.changed.member);
         break;
 
      case dveMotifRecipientListChangeTagAction:
         sprintf(tmp, "Triggered member #%d\n",cbs->arg.action.member);
         break;
 
      default:
         sprintf(tmp, "Unknown callback tag");
         return;
   }
   SetFooter(tmp);
}

/*
 * List members of the participant lists.
 */
void recipTestListMembers(void)
{
   char tmp[50];
   int x = 0;

   memset(tmp, '\0', strlen(tmp));

   /*
    * Display the userHandles of those in the multi-select
    * conference participant list.
    */
   strcpy(tmp, "Multiple:"); x = strlen(tmp);
   {
      int i;
      for (i=0; i<DVE_MAX_PARTICIPANTS; i++)
      {
         if (DvePartListIsSet(i,&partListM))
         {
            sprintf(tmp + x, " #%d",i); x = strlen(tmp);
         }
      }
   }
 
   /*
    * dveMotifRecipientListGetSelection() is used to obtain the user
    * handle of the currently selected participant of a browse-select
    * list.
    */
   sprintf(tmp + x, " Browse: #%d", dveMotifRecipientListGetSelection(listB));
   SetFooter(tmp);
}

/*
 * Select all conference participants of a multi-select
 * DveMotifRecipientList.
 */
void recipTestSelectAll(void)
{
   dveMotifRecipientListSelectAll(listM);
}
 
/*
 * Deselect all conference participants from a multi-select
 * DveMotifRecipientList.
 */
void recipTestDeselectAll(void)
{
   dveMotifRecipientListDeselectAll(listM);
}

/*
 * Orderly shutdown of the OpenDVE session.
 */
void
QuitPlugin(DveSession dveSession)
{
   dveCloseSession(dveSession);
 
#ifdef XT
   dveUnregisterWithXt(dveSession);
#endif
 
#ifdef XVIEW
   dveUnregisterWithXView(dveSession);
#endif
   exit(0);
}

/*
 * QuitPlugin if requested.
 */
static void
PluginControlMsg(DveSession dveSession, DveUserHandle sender,
                 DveDataPacket* packet)
{
   switch ((DvePluginManagementMsgTag)packet->tag)
   {
      case dvePluginManagementMsgTagQuit:
         QuitPlugin(dveSession);
         break;
      case dvePluginManagementMsgTagShow:
         ShowPlugin();
         break;
      case dvePluginManagementMsgTagHide:
         HidePlugin();
         break;
   }
}

void
HidePlugin()
{
   XtPopdown(recipTestAppShell);
}
 
void
ShowPlugin()
{
   XtPopup(recipTestAppShell, XtGrabNone);
}
 
void
introduceYourself()
{
   /*
    * Start up OpenDVE.
    */
   if (!(dveSession=dveOpenSession(RECIPIENT_PLUGIN_PORT)))
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }

   dveSetUserCallback(dveSession,dvePluginManagementProtocol, PluginControlMsg);

   /*
    * Establish the procedures that are to be called when
    * an OpenDVE event occurs.  The events of interest in the example are
    * Starting, State, NewMember, MemberLeft, MemberAccepted,
    * and EngineShutdown.
    */
   dveSetEventCallback(dveSession,dveEventTagStarting,starting);
   dveSetEventCallback(dveSession,dveEventTagState,memberListChanged);
   dveSetEventCallback(dveSession,dveEventTagNewMember,memberListChanged);
   dveSetEventCallback(dveSession,dveEventTagMemberLeft,memberListChanged);
   dveSetEventCallback(dveSession,dveEventTagMemberAccepted,memberListChanged);
   dveSetEventCallback(dveSession,dveEventTagEngineShutdown,memberListChanged);

   /*
    * Register this event processing with the Xt toolkit.  Without this
    * call OpenDVE events will not be received.
    */
   if (dveRegisterWithXt(dveSession,UxAppContext)<0)
   {
      fprintf(stderr, "Unable to register with Xt loop.");
      dveCloseSession(dveSession);
      exit(0);
   }

   dveReportPresence(dveSession);
}

/*
 * Set the footer to show the action has taken place.
 */
void
SetFooter(const char *msg)
{
   XmString str=XmStringCreateSimple((char*)msg);
   XtVaSetValues(label1,
                 XmNlabelString,str,
                 NULL);
   XmStringFree(str);
}
