/* ISvideoControlShm.h 
 * Defines the structure for two-way communication between the control panel
 * process and the capture process. Also has control fields to do A/V sync.
 */

#ifndef _VIDEO_SHM_CONTROLS_
#define _VIDEO_SHM_CONTROLS_

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <dve/dve.h>
#include <dve/mm/shm.h>


/* Parameter codes for most of the parameters in shared memory that the
 * control panel / capture process is allowed to touch
 * Important : Do *not* change these values arbitrarily!
 */
#define DVE_FrameRate                0
#define DVE_Bandwidth                1
#define DVE_Quantization             2
#define DVE_Brightness               3
#define DVE_Contrast                 4
#define DVE_Hue                      5
#define DVE_Saturation               6
#define DVE_Sharpness                7
#define DVE_FrameSize                8
#define DVE_FrameHeight              9
#define DVE_FrameWidth              10
#define DVE_Quality                 11
#define DVE_Input                   12
#define DVE_Format                  13
#define DVE_DoGamma                 14
#define DVE_DoCalibration           15
#define DVE_DoRefresh               16
#define DVE_HardwareDevice          17
#define DVE_Compression             18
#define DVE_Favor                   19

#define DVE_maxParameters           20

/* AV sync structure */
typedef struct {
  int32  audioVideoSyncThreshold;
  
  int32 localAudioTimeStampSec;
  int32 localAudioTimeStampUSec;
  int32 localVideoTimeStampSec;
  int32 localVideoTimeStampUSec;
  
  int32 audioTimeStampSec[DVE_MAX_PARTICIPANTS];
  int32 audioTimeStampUSec[DVE_MAX_PARTICIPANTS];
  int32 videoTimeStampSec[DVE_MAX_PARTICIPANTS];
  int32 videoTimeStampUSec[DVE_MAX_PARTICIPANTS];
} DveAVSyncControls;


/* Video and AV sync control structure 
 */
typedef struct {
  /* Filled in by the control panel for the capture process */
  u_int32 requestedValues[DVE_maxParameters];

  /* Flags that will inform the capture process that shared memory values 
   * have changed 
   */
  int32 changeFlagsRequested[DVE_maxParameters];

  /* Filled in by the capture process for the control panel */
  u_int32 reportedValues[DVE_maxParameters];

  /* Flags that will inform the control panel that shared memory values 
   * have changed 
   */
  int32 changeFlagsReported[DVE_maxParameters];


  /* Error/status return from the capture process. Note that the error
   * returned must be a value that the control panel can interpret
   */
  int32 errorReturn;
  
  /* Fatality of the error. This is set to FATAL_ERROR if the capture 
   * process will terminate 
   */
  int32 fatalError;

  /* Running Status of the capture process. The capture process and the
   * Control panel co-operate through this field.
   */
  int32 runningStatus;


  /* A/V sync controls */
  DveAVSyncControls avSyncControls;

  /* Extension slot : if a capture process so desires, it can allocate its 
   * own extended shared memory and store its id here 
   */
  int32 extensionShmId;

} DveVideoShmControls;


/* Possible values for the runningStatus field */
#define DVE_STOPPED    0
#define DVE_ACTIVE     1
#define DVE_PAUSED     2
#define DVE_IDLE       3

/* Possible values for reported hardware device */
#define CONFERENCE_CAM     0
#define PARALLAX           1
#define SUNVIDEO           2
#define IBMSUNRISE         3
#define J300               4

#define DVE_NUM_BOARDS     5

/* Possible values for format */
#define H261               0
#define JPEG               1
#define ICLB               2
#define DVE2               3
#define MPEG               4
#define MJPG               5
#define J411               6
#define J422               7
#define IP64               8

#define DVE_NUM_FORMATS    9

/* Possible values for error status : anything other than these values is a
 * board-specific value
 */
/* Could not open the video device */
#define DVE_ERROR_OPENFAILED     1

/* Was able to open the video device, but could not start video transmission */
#define DVE_ERROR_STARTFAILED    2

/* EOF returned by video device from some reason */
#define DVE_ERROR_EOFRETURNED    3

/* Error occurred setting camera parameters */
#define DVE_ERROR_SETPARAMETERS  4

/* Error occurred reading from the camera */
#define DVE_ERROR_READ           5

/* No error */
#define DVE_NOERROR              0


#define NONFATAL_ERROR    0
#define FATAL_ERROR       1


#define BUFSIZE 256000
#define SHM_KEY 6805


#ifdef __cplusplus
}
#endif

#endif
