/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Declares the InSoft profile management library.  Conceptually, a profile
 *   is a two-dimensional associative array of strings.  The index for the
 *   first dimension is called a section; the index for the second is called
 *   a tag; thus, an entry is filed as if it were profile[section][tag].
 *   Of course, without C++, we don't get such convenient syntax.  (This file
 *   includes a C++ definition for a class called ISProfileObject, which
 *   provides such an array; it has not been tested, and is likely to be
 *   highly inefficient unless it gets further work--it has no provision for
 *   reading or writing multiple items at a time.)
 *  An entry in the profile is referred to as an item.  The item contained
 *   in profile[section][tag] is referred to as [section:tag] in the comments
 *   in this header.  This appears to be some vestige of VMS syntax rearing
 *   its head--surprising; I'd thought I'd suppressed those memories.
 *  A profile is locked (with ordinary Unix file locking) while it is being
 *   read or written.
 *  Most of these functions return -1 to indicate an error.  Which error is
 *   indicated in the variable ispErr, whose enum values are declared below.
 *
 * Implemented by:
 *  ISprofile.c
 *
 * Classes:
 *  ISProfileItem -- helper class for ISProfileObject
 *  ISProfileSection -- helper class for ISProfileObject
 *  ISProfileObject -- associative array.
 *
 * Types:
 *  ISprofile -- thunk (pointer to undefined struct) for a profile
 *  IspErr -- enum of error codes
 *
 * Variables:
 *  ispErr -- most recent error result
 *
 * Author:
 *  francis, 6 Apr 1994
 *
 * Notes:
 *  The file format for profiles is (intended to be) the same as the Windoze
 *   .INI format, with sections headed up with [<section-name>] and comments
 *   marked with ;.  Major departure: a file may have multiple headings for
 *   the same section; indeed, under some circumstances, this library may
 *   create such files.  This was a deliberate decision, for a mix of
 *   efficiency and flexibility.  We'll need to give users a command-line
 *   utility to merge sections together.
 *  There are three files for each profile: locked, prefs, and defaults.
 *   Entries in locked take precedence over entries in prefs, which take
 *   precedence over entries in defaults.  By convention, a component called
 *   foo, installed in directory bar, will put locked in bar/foo_locked,
 *   prefs in ~/CommuniqueSave/foo, and defaults in bar/foo.  The result
 *   is that bar/foo contains system-wide defaults, ~/CommuniqueSave/foo
 *   contains the user's own preferences, and bar/foo_locked contains
 *   locked settings--i.e., settings that the user cannot override with hir
 *   prefs.
 *  A program should also provide default values (via ispSetDefault()) that
 *   are used if a value is not found in any of the files.
 *
 * Change Log:
 *  $Log$
 */
#ifndef _INSOFT_PROFILE_MANAGER_WITH_SECTIONS_H_
#define _INSOFT_PROFILE_MANAGER_WITH_SECTIONS_H_

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

#ifdef _INSOFT_PROFILE_MANAGER_H_
#error The same executable cannot use both the old and the new ISprofile lib
#endif /*_INSOFT_PROFILE_MANAGER_H_*/

#ifdef __cplusplus
extern "C" {
#endif

/* Thunk for the profile */
typedef struct _ISprofile* ISprofile;

/* Returns a thunk referring to a profile whose defaults are in
 * defaultsFile and whose user prefs are in prefsFile.  The name
 * strings are copied.
 */
ISprofile ispMakeProfile(const char* defaultsFile,
			 const char* lockedFile,
			 const char* prefsFile);
/* Wrapper around ispMakeProfile().  fileName is the final filename of
 * the prefs & defaults file; for the locked file, it's
 * strcat(fileName,"_locked").  The prefs file is in userSaveDir (which can
 * begin with ~); the system-wide files are in systemDir (which can't).
 */

ISprofile ispMakeUserProfile(const char* fileName,
			     const char* userSaveDir,
			     const char* systemDir);

/* Very small wrapper around ispMakeUserProfile()--this is the one you'd
 * normally use.  It just invokes ispMakeUserProfile(fileName,
 * "~/CommuniqueSave",systemDir).
 */
ISprofile ispMakeStdProfile(const char* fileName,
			    const char* systemDir);

/* Shuts down the given profile: frees associated memory, unlocks & closes
 * any open files (shouldn't be any), etc.
 */
void ispFreeProfile(ISprofile profile);

/* Set the hard-coded default (which is used as a fallback if the item
 * is not in any file) for [section:tag] to string.  The string is
 * copied.  Returns -1 on failure, 0 on success.
 */
int ispSetDefault(ISprofile profile,
		  const char* section,
		  const char* tag,
		  const char* string);
/* Get the hard-coded default for [section:tag].  Returns a pointer to
 * the internal copy of the default string; don't modify this.  Returns
 * NULL on failure.
 */
const char* ispGetDefault(ISprofile profile,
			  const char* section,
			  const char* tag);
/* Clear the hard-coded default for [section:tag].  Not sure why you'd
 * need to do this, but hey.  Returns 1 if the item does not exist, -1
 * on other error, 0 on success.
 */
int ispClearDefault(ISprofile profile,
		    const char* section,
		    const char* tag);

/* Reads the item [section:tag] from profile, into buff, which holds
 * up to buffsize bytes.  Returns length of the string, or -1 on error.
 * If string is too long for buffer, returns actual length, but truncates
 * to buffer (does 0-terminate).  (Does not count the 0 in the length
 * returned.)
 */
int ispReadString(ISprofile profile,
		  const char* section,
		  const char* tag,
		  char* buff,
		  int buffsize);

/* Convenience wrapper around ispReadString(); returns a pointer to a
 * static internal buffer, which holds at least 512 bytes.  Returns
 * NULL on error (including "string too long for internal buffer").
 */
const char* ispGetString(ISprofile profile,
			 const char* section,
			 const char* tag);
/* Writes string into the item [section:tag] in profile.  Returns 0
 * on success, or -1 on error.  ispWriteString(profile,section,tag,NULL)
 * is equivalent to ispClearString(profile,section,tag) (below).
 */
int ispWriteString(ISprofile profile,
		   const char* section,
		   const char* tag,
		   const char* string);
/* Clears the item [section:tag] from the prefs file (which reverts it
 * to the default values).
 */
int ispClearString(ISprofile profile,
		   const char* section,
		   const char* tag);

/* Similar to XtVa{Get,Set}Values(); more efficient than repeated uses
 * of isp{Read,Write}String(), which have to open & scan the file each time.
 */
/* varargs is a NULL-terminated sequence of 
 * (const char* section, const char* tag, char* buff, int* nbytes)
 * (*nbytes) is an in-and-out parameter--when passed it, it contains
 * the max # of bytes; when returned, it contains the actual (or -1 if
 * not found).
 * Function returns -1 on error, or # of items found.
 */
int ispVaReadStrings(ISprofile profile,...);
/* varargs is a NULL-terminated sequence of 
 * (const char* section, const char* tag, const char* str)
 * Returns -1 on error, or 0 on success.
 */
int ispVaWriteStrings(ISprofile profile,...);

/* Check to see if given strings are locked.  varags is a NULL-terminated
 * sequence of
 * (const char* section, const char* tag, int* locked)
 * (*locked) returns the lockedness (!=0 means true) of the string.
 * Returns -1 on error, or # of locked strings.
 */
int ispVaCheckLocks(ISprofile profile,...);

/* Returns 1 if [section:tag] is "yes", 0 if it's "no" (case-insensitive),
 * -1 on error. */
int ispReadFlag(ISprofile profile,
		const char* section,
		const char* tag);
/* Sets [section:tag] to (flag ? "yes" : "no"); returns -1 on error, 0
 * on success. */
int ispWriteFlag(ISprofile profile,
		 const char* section,
		 const char* tag,
		 int flag);

/* Returns 1 if [section:tag] is locked, 0 if unlocked, -1 on error */
int ispCheckLock(ISprofile profile,
		 const char* section,
		 const char* tag);

/* Delete the given section from the prefs file.  Returns 0 on success,
 *  <0 on error.  Note that ("section doesn't exist") is not a problem.
 */
int ispDelSection(ISprofile profile,
		  const char* section);

/* Get all sections in the profile.  Returns a null-terminated array of
 *  strings (const char*s).  This array should be freed with
 *  ispFreeSections(), below, partly because it's easier than writing the
 *  loop yourself, but mostly because it may not have been allocated with
 *  simple malloc().
 */
const char** ispGetSections(ISprofile profile);

/* Free an array of sections returned from ispGetSections() */
void ispFreeSections(const char** sections);

/* Get all tags in the section.  Returns a null-terminated array of
 *  strings (const char*s).  This array should be freed with
 *  ispFreeTags(), below, partly because it's easier than writing the
 *  loop yourself, but mostly because it may not have been allocated with
 *  simple malloc().
 */
const char** ispGetTags(ISprofile profile,
			const char* section);

/* Free an array of tags returned from ispGetTags() */
void ispFreeTags(const char** tags);

/* A convenient way to set multiple strings at once (i.e., without opening
 *  the file multiple times), by batching up the I/O.
 */
typedef struct _IspWriteBatch* IspWriteBatch;

/* Creates a write batch for writing to the given profile */
IspWriteBatch ispWBOpen(ISprofile profile);

/* Perform all the batched-up writes.  This clears the batch; it can now
 *  be used again.
 */
int ispWBFlush(IspWriteBatch batch);

/* Clear out all the batched-up writes, without writing to the disk */
int ispWBClear(IspWriteBatch batch);

/* Flush the writes & close the batch (clean up its memory & stuff).  To
 *  abort and close, use ispWBClear() first.
 */
int ispWBClose(IspWriteBatch batch);

/* Add a write to the batch.  The strings section, tag, and string are
 *  not duplicated; they must not be freed or overwritten prior to flushing
 *  the batch.
 */
int ispWBSetString(IspWriteBatch batch,
		   const char* section,
		   const char* tag,
		   const char* string);

/* A convenient way to read multiple strings at once (i.e., without opening
 *  the file multiple times), by batching up the I/O.
 */
typedef struct _IspReadBatch* IspReadBatch;

/* Creates a read batch for reading the given profile */
IspReadBatch ispRBOpen(ISprofile profile);

/* Perform all the batched-up reads.  Strings read will be stored into
 *  the buffers established with ispRBReqString(), below.  Returns
 *  # of strings found, or <0 on error.
 */
int ispRBRead(IspReadBatch batch);

/* Close the batch (clean up its memory & stuff).  If freeBuffers is
 *  true, frees the buffers created by ispRBReqString().
 */
int ispRBClose(IspReadBatch batch);

/* Add a request to the batch.  The strings section and tag are not
 *  duplicated; they must not be freed or overwritten prior to reading
 *  the batch.  The request created will have a buffer of maxbytes bytes
 *  allocated; when read, the string (including the 0 terminator) will be
 *  stored in this buffer.
 */
int ispRBReqString(IspReadBatch batch,
		   const char* section,
		   const char* tag,
		   int maxbytes);

/* Get a string read into the batch by ispRBRead(), above */
const char* ispRBGetString(IspReadBatch batch,
			   const char* section,
			   const char* tag);

typedef enum
{
  ispErrOK=0,
  ispErrOutOfMemory,
  ispErrBadArg,
  ispErrNoHome,
  ispErrNotFound,
  ispErrNoFile,
  ispErrNoPerms,
  ispErrInternal,
  ispErrFileIsDir,
  ispErrCantLock,
  ispErrCantUnlock,
  ispErrUnknown,
  ispErrUseErrno
  } IspErr;

extern IspErr ispErr;

#ifdef __cplusplus

class ISProfileItem {
  ISprofile profile;
  const char* section;
  const char* tag;
public:
  ISProfileItem(ISprofile _profile,
		const char* _section,
		const char* _tag)
    : profile(_profile),
      section(_section),
      tag(_tag)
	{}

  const char* operator=(const char* str)
    {
      ispWriteString(profile,section,tag,str);
      return str;
    }

  operator const char*(void)
    {
      return ispGetString(profile,section,tag);
    }
};

class ISProfileSection {
  ISprofile profile;
  const char* section;
public:
  ISProfileSection(ISprofile _profile,
		   const char* _section)
    : profile(_profile), section(_section)
      {}

  ISProfileItem operator[](const char* tag)
    {return ISProfileItem(profile,section,tag);}
};

class ISProfileObject {
  ISprofile profile;
public:
  ISProfileObject(const char* fileName,
		  const char* systemDir)
    {
      profile=ispMakeStdProfile(fileName,systemDir);
    }
  ~ISProfileObject() {ispFreeProfile(profile);}

  /* For the more sophisticated stuff, we need to be able to get at the
   * profile thunk.
   */
  operator ISprofile() {return profile;}

  ISProfileSection operator[](const char* section)
    {return ISProfileSection(profile,section);}
};
  
#endif /*__cplusplus*/

#ifdef __cplusplus
}
#endif

#endif /*_INSOFT_PROFILE_MANAGER_WITH_SECTIONS_H_*/
