#ifndef _DVE_AV_H_
#define _DVE_AV_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/AVCtl/core.h>
#include <dve/AVCtl/event.h>

/******************** Functions to set up a session **********************/
/*
 * dveAVOpenSession
 *   Open a new connection to the AV Server. 
 * Input : mode : Specifies the mode of the connection
 *          dveAVCtlConnectionModeFull : Be able to both send and receive 
 *                                       audio and video
 *          dveAVCtlConnectionModeReceive : Receive audio and video only.
 *         session : An initialized DveSession created by the application.
 *                   It is probably a good idea to set up your OpenDVE 
 *                   callbacks by the time you call dveAVOpenSession(), since
 *                   any queued-up OpenDVE events will be processed in this
 *                   call.
 *         negotiate : Should the engine do negotiation for sending and
 *                     receiving?
 *
 * 
 * Output : 
 * Return : Returns a DveAVSession, a thunk to be used in all subsequent
 *          DveAV calls, zero on error
 * Notes :
 */
DveAVSession dveAVOpenSession(DveAVConnectionMode mode,
			      DveSession session,
			      int negotiate);


/*
 * dveAVCloseSession
 *     Close an existing session. This will stop all audio/video functions
 *      that were initiated on your behalf.
 * Input : session : The session returned by dveAVOpenSession()
 * Output :
 * Return :
 * Notes :
 */
void dveAVCloseSession(DveAVSession session); 

/*
 * dveAVGetAvailMedia
 *   Convenience function that returns information about what media are
 *   available on the host machine. domains[i] will be set to a particular 
 *   domain that is available (audio or video). sendModes[i] will then 
 *   indicate whether that domain is available for sending, or receiving, or
 *   both. For instance, if domains[0] is set to dveAVDomainVideo, it indicates
 *   that video is available; if sendModes[0] is set to dveAVSendModeAll, then
 *   the endpoint can both send and receive video. If sendModes[0] is 
 *   dveAVSendModeReceive, that means that the endpoint can receive but not
 *   send video (which would be the case if the endpoint has no video 
 *   hardware).
 *
 * Input : session : The session returned by dveAVOpenSession()
 * Output : domains : An array of available domains (audio/video). 
 *          sendModes : An array of corresponding send modes.
 * Return : Number of available domains. 
 * Notes :
 */
int dveAVGetAvailMedia(DveAVSession session, DveAVDomain *domains, 
		       DveAVSendMode *sendModes);

/******************* Functions to manipulate members *************************/

/* A plugin that has opened a connection with negotiation will need to use
 * only dveAVEnableMember() and dveAVDisableMember(). If we are doing
 * negotiation, the list of available members is exactly the list of members 
 * in the current conference; also, a member index is really the user-handle 
 * of a member in the conference.
 * 
 * If you are writing an application that broadcasts audio and video using
 * multicast, then you can set up a "dummy" member that sends/receives audio
 * and video over a multicast socket.
 */

/*
 * dveAVAddMember
 *   Add a member to send video to/receive video from. If the server is doing
 *   negotiation, this call is *invalid* : the list of available members and
 *   other related information is obtained through negotiation only. If you
 *   are not doing negotiation, you *must* use this call for each stream you
 *   send out/receive.
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         info : Member information. This could be NULL. You would pass in
 *                NULL if, for example, you wanted to broadcast audio/video
 *                using multicast, or receive audio/video from a multicast
 *                stream. 
 *
 * Output :
 * Return : A positive integer, that represents the "member index" for this
 *          member, to be used to identify this member in subsequent calls.
 *          Returns -1 on error.
 * Notes :
 */
int dveAVAddMember(DveAVSession session, DveAVMemberInfo *info);

/*
 * dveAVRemoveMember
 *   Remove a member added using dveAVAddMember(). This will stop all sending
 *   and receiving audio/video functions with respect to this member. Again, 
 *   this call is invalid if you are doing negotiation, members are then 
 *   dropped automatically when they drop out of a conference.
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         memberIndex : Index returned by dveAVAddMember()
 * Output :
 * Return :
 * Notes :
 */
void dveAVRemoveMember(DveAVSession session, int memberIndex);

/*
 * dveAVEnableMember
 *    Enable sending/receiving functions for a member. If we enable sending
 *    functions for a member and we are already transmitting video, then 
 *    video is transmitted to the new member also, provided transport is set
 *    using setTransport(). Enabling receiving functions for a member has the
 *    effect of starting an appropriate audio/video viewer for that member. 
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         memberIndex : Index returned by dveAVAddMember(), or openDVE 
 *                       user-handle if doing negotiation.
 *         domain : dveAVDomainAudio, dveAVDomainVideo or dveAVDomainAudioVideo
 *         mode : dveAVSend, dveAVReceive, or dveAVAll.
 * 
 * Output :
 * Return : 
 * Notes : By default, neither audio nor video is enabled for sending or
 *         receiving.
 */
void dveAVEnableMember(DveAVSession session, int memberIndex, 
		       DveAVDomain domain,
		       DveAVSendMode mode);

/*
 * dveAVDisableMember
 *   Disable sending/receiving functions for a member. Disabling receiving 
 *   functions for a member has the following effect : If we are already
 *   receiving data from the member for the specified domain(s), the viewer
 *   is stopped (killed). Also, all further feeds from this member are ignored 
 *   until receiving functions are enabled again. Disabling sending functions
 *   for a member has the following effect : If we are sending a unicast feed
 *   to this member for the specified domain, the feed is stopped. If we are
 *   transmitting multicast, however, then this call has *no effect*. If you
 *   are transmitting multicast, you should disable receiving functions on the
 *   receiving side rather than disable sending functions on the sending side.
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         memberIndex : Index returned by dveAVAddMember(), or openDVE
 *                       user-handle if doing negotiation.
 *         domain : dveAVDomainAudio, dveAVDomainVideo or dveAVDomainAudioVideo
 *         mode : dveAVSend, dveAVReceive, or dveAVAll.
 * 
 * Output :
 * Return :
 * Notes :
 */
void dveAVDisableMember(DveAVSession session, int memberIndex, 
			DveAVDomain domain,
			DveAVSendMode mode);
/*
 * dveAVSetTransport
 *   Set sending/receiving transport information for a member. If you 
 *   set sending transport for a member and you are already sending to this
 *   member, then the stream is redirected to use the new transport. Setting
 *   receiving transport for a member when we are already receiving from this
 *   member is considered illegal at the present time, and the transport info
 *   is ignored.
 * 
 * If you are not doing negotiation, then you must set sending and receiving
 * transport for each member in order to receive audio/video from him/her.
 * 
 * If the engine is doing negotiation on your behalf, you will not need to use
 * this call at all unless you wish to explicitly specify port numbers to 
 * receive video on.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         memberIndex : Index returned by dveAVAddMember(), or openDVE
 *                       user handle if doing negotiation. 
 *         domain : dveAVDomainAudio or dveAVDomainVideo
 *         mode : dveAVSendModeSend or dveAVSendModeReceive. If you are 
 *                doing negotiation, dveAVSendModeSend is invalid, because of
 *                the way the negotiation paradigm works.
 *         transport : An appropriately filled structure containing transport 
 *                     details for this member.
 * 
 * Output : transport : On successful return, will contain actual transport
 *          info for this member. In particular, if the engine was instructed
 *          to bind on an arbitrary port, then the port actually bound on will
 *          be reported.
 * 
 * Return : dveAVOK if successful, appropriate error status otherwise
 * Notes :
 */
DveAVError dveAVSetTransport(DveAVSession session, int memberIndex, 
			     DveAVDomain domain, DveAVSendMode mode,
			     DveAVTransport *transport);


/******** The following functions are used to send audio and video *********/


/*
 * dveAVStart
 *   Start sending audio, video or both. Note that prior to this call a list
 *   of members to send to must have already been set up using the various
 *   AddSendingMember() calls. Also, if we aren't doing negotiation, transport
 *   information must have been set up for all members that we are sending to.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainAudioVideo.
 *
 * Output :
 * Return :
 * Notes :
 */
void dveAVStart(DveAVSession session, DveAVDomain domain);


/*
 * dveAVStop
 *   Stop sending audio, or video, or both.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainAudioVideo.
 *
 * Output :
 * Return :
 * Notes :
 */
void dveAVStop(DveAVSession session, DveAVDomain domain);


/*
 * dveAVPause
 *   Pause(Mute)/Resume(Unmute) outgoing audio, or video, or both.
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainAudioVideo.
 *         pause : Video is paused if True, resumed otherwise
 * 
 * Output :
 * Return :
 * Notes :
 */
void dveAVPause(DveAVSession session, DveAVDomain domain, int pause);

/*
 * dveAVStartMonitor
 *   Starts an audio/video local monitor. This call always applies to the 
 *   currently active audio/video board and compression format.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainAudioVideo (which will start both audio and video
 *                                  monitors).
 * Output :
 * Return :
 * Notes :
 */
void dveAVStartMonitor(DveAVSession session, DveAVDomain domain);

/*
 * dveAVStopMonitor
 *    Stops an audio/video local monitor. This call always applies to the 
 *    currently active audio/video board and compression format.
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainAudioVideo (which will stop both audio and video
 *                                  monitors).
 * Output :
 * Return :
 * Notes :
 */
void dveAVStopMonitor(DveAVSession session, DveAVDomain domain);


/***** The following functions manipulate various audio/video parameters *****/


/*
 * dveAVSetParameter
 *   Set an audio, video, or transport parameter. If the parameter is an 
 *   the audio or video domain, it affects only the audio or video device
 *   specified as part of the argument. If it is in the transport domain, 
 *   it will remain in effect until set again.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainTransport
 *         tag : A domain-sepcific parameter tag.
 *         size : Size of the parameter 
 *         value : Pointer to actual value
 *
 * Output :
 * Return :
 * Notes : Most audio, video and transport parameters are of type unsigned int
 */
DveAVError dveAVSetParameter(DveAVSession session, DveAVDomain domain,
			     unsigned int tag, int size, void *value);


/*
 * dveAVSetIntegerParameter
 *   Convenience function; Wrapper around dveAVSetParameter(). Sets the 
 *   value of an integer-valued parameter.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or dveAVDomainTransport
 *         tag : Domain-specific tag.
 *         value : Value to be set
 *
 * Output :
 * Return : dveAVOK on success 
 *          dveAVErrorIllegalTag :  Tag is undefined for specified domain
 *          dveAVErrorIllegalDomain : Illegal domain
 *          dveAVErrorUndefined : Tag is undefined for specified device
 *          
 * Notes : 
 */
DveAVError dveAVSetIntegerParameter(DveAVSession session, DveAVDomain domain,
				    unsigned int tag, unsigned int value);

/*
 * dveAVGetParameter
 *   Get the value of an audio, video, or transport parameter. The "value" 
 *   parameter is both an input and output parameter. For most tags this is
 *   a struct of some sort that contains a boardId or deviceId field. This
 *   must be set to the board whose value you are interested in.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainTransport
 *         tag :  A domain-sepcific parameter tag.
 *         size : Size of the space pointed to by "value"
 *         value : Must be filled out with the id of the video board whose
 *                 value you are interested in.
 *
 * Output : value : Preallocated pointer; Filled with value on return
 * Return : dveAVOK on success 
 *          dveAVErrorIllegalTag :  Tag is undefined for specified domain
 *          dveAVErrorIllegalDomain : Illegal domain
 *          dveAVErrorUndefined : Tag is undefined for specified device
 *          
 *
 * Notes :
 */
DveAVError dveAVGetParameter(DveAVSession session, DveAVDomain domain,
			     unsigned int tag, int size, void *value);


/*
 * dveAVGetIntegerParameter
 *   Convenience function : Wrapper around dveAVGetParameter() to get the
 *   value of an integer-valued parameter. This call pertains to the currently
 *   active audio/video device only.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or 
 *           dveAVDomainTransport
 *
 * Output : value : filled with value on success
 * Return : dveAVOK on success 
 *          dveAVErrorIllegalTag :  Tag is undefined for specified domain
 *          dveAVErrorIllegalDomain : Illegal domain
 *          dveAVErrorUndefined : Tag is undefined for specified device
 *          
 *
 * Notes :
 */
DveAVError dveAVGetIntegerParameter(DveAVSession session, 
				    DveAVDomain domain,
				    unsigned int tag, unsigned int *value);


/*
 * dveAVGetReportedParameter
 *   Gets the value of a parameter reported by an audio/video capture driver.
 *   This value is distinct from the value returned by dveAVGetParameter(),
 *   which is the value set by the interface (us) for the capture driver.
 *   In contrast to dveAVGetParameter(), this call always applies to the
 *   currently active board, since only one audio/video board is allowed
 *   to capture data at one time. 
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or dveAVDomainTransport
 *         tag : Domain-specific tag
 *         size : Size of memory pointed to by "value"
 *
 * Output : value : Pointer to a tag-specific struct that will contain the 
 *              value, on success
 *
 * Return : dveAVOK on success, dveAVErrorCaptureInactive if audio/video
 *          capture is currently inactive
 *
 * Notes :
 */
DveAVError dveAVGetReportedParameter(DveAVSession session, DveAVDomain domain,
				     unsigned int tag, int size, void *value);


/*
 * dveAVGetChangedReportedParameters
 *   Gets all capture-driver reported parameters that are currently "dirty"
 *   (ie unread since the last call to this function). The domain specifies
 *   whether we are interested in audio, video, or transport parameters. 
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, or dveAVDomainTransport
 *   
 * Output : tags : On return, this will point to an array of tags that have
 *                 changed.
 *          values : On return, values[i] will point to the domain-specific 
 *                   structure containing the value for tags[i].
 * 
 * Return : Number of changed parameters; < 0 on error.
 * Notes : tags and values must be freed by a subsequent call to 
 *         dveAVFreeChangedParameters()
 */
int dveAVGetChangedReportedParameters(DveAVSession session,
				      DveAVDomain domain,
				      unsigned int **tags,
				      void **values);

/*
 * dveAVFreeChangedParameters
 *   Frees the info returned by dveAVGetChangedReportedParameters()
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         ntags : Number of tags
 *         tags : Array of tags
 *         values : Array of domain-specific values
 *
 * Output :
 * Return :
 * Notes :
 */
void dveAVFreeChangedParameters(DveAVSession session,
				int ntags, unsigned int *tags,
				void *values);

/********** Functions to show and hide board-specific interfaces *********/

/*
 * dveAVShowBoard
 *   Display the board-specific interface, if one exists
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, dveAVDomainAudioVideo
 *                  
 * Output :
 * Return :
 * Notes :
 */
void dveAVShowBoard(DveAVSession session, DveAVDomain domain);

/*
 * dveAVHideBoard
 *   Hide (unmap) the board-specific interface, if one exists
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, dveAVDomainAudioVideo
 *                  
 * Output :
 * Return :
 * Notes :
 */
void dveAVHideBoard(DveAVSession session, DveAVDomain domain);

/***************** Functions to register and remove callbacks ***************/

/* Callbacks can be set up for any of the AV Events described in event.h. */


/*
 * dveAVAddEventCallback
 *   Add a function to the callback list associated with the given event tag.
 *    
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio or dveAVDomainVideo.
 *         tag : event tag
 *         callback : Callback function to be called
 *         userData : User data to be passed to callback function
 *
 * Output :
 * Return : Callback id for this callback, to be used when removing it from 
 *          the callback list
 *
 * Notes :
 */
int dveAVAddEventCallback(DveAVSession session,
			  DveAVDomain domain,
			  DveAVEventTag tag,
			  DveAVEventCallback callback,
			  void *userData);

/*
 * dveAVRemoveEventCallback
 *   Remove a callback function for a specified event tag.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio or dveAVDomainVideo.
 *         tag : Event tag
 *         callbackId : callback id returned by dveAVAddEventCallback()
 * 
 * Output :
 * Return :
 * Notes :
 */
void dveAVRemoveEventCallback(DveAVSession session, 
			      DveAVDomain domain, int tag, int callbackId);

/********************** Arbitration functions ****************************/
/* The AVEngine also acts as a device arbitrator between OpenDVE plugins. 
 *  Multiple clients can connect to it, and ask to lock a particular device. 
 * When a client locks a device, it owns it - the engine rejects lock requests
 * from other clients for the same device. Having used the device, a client
 * frees it (or it gets free when the client dies), and other clients can 
 * now use the device. When you start capture, you automatically lock the 
 * device, if it's available; and unlock it when you stop capture.
 */

/*
 * dveAVLock
 *   Lock an audio/video device.
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio or dveAVDomainVideo.
 *         id : Domain-specific board id
 *
 * Output :
 * Return : dveAVOK on no error, dveAVErrorDeviceBusy, dveAVErrorOpen
 * Notes :
 */
DveAVError dveAVLock(DveAVSession session, DveAVDomain domain, u_int32 id);

/*
 * dveAVUnlock
 *   Unlock a device that was previously locked.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio or dveAVDomainVideo.
 *         id : Domain-specific board id
 *
 * Output :
 * Return : dveAVOK on no error
 * Notes :
 */
DveAVError dveAVUnlock(DveAVSession session, DveAVDomain domain, u_int32 id);

/*
 * dveAVUnlockAll
 *   Unlocks all devices currently locked by us.
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio, dveAVDomainVideo, dveAVDomainAudioVideo
 * Output :
 * Return : dveAVOK on no error
 * Notes :
 */
DveAVError dveAVUnlockAll(DveAVSession session, DveAVDomain domain);


/***************** Session Management Functions *************************/


/*
 * dveAVSetFds
 *   Fills out a file descriptor set that needs to be monitored by the 
 *   application. This is used by line mode apps. An application adds its
 *   own file descriptors to the fd_set after this call, does a select(),
 *   and then calls dveAVProcessFds() to handle activity on any of the
 *   file descriptors.
 * Input : session : The session returned by dveAVOpenSession()
 *         readfds : File descriptor set to fill out.
 * Output :
 * Return :
 * Notes :
 */
int dveAVSetFds(DveAVSession session, fd_set *readfds);


/*
 * dveAVProcessFds
 *   Examines the set of file descriptors in which select() has found 
 *   activity and responds to activity on OpenDVE and OpenDVEAV file 
 *   descriptors.
 * Input :
 * Output :
 * Return :
 * Notes :
 */
void dveAVProcessFds(DveAVSession session, fd_set *readfds);

/*
 * dveAVSimpleLoop
 *   Initiates an event loop for non-interactive applications. This call does
 *   not return until the session is closed, and will take over the 
 *   operation of the program by signalling all of the registered callbacks.
 * Input :
 * Output :
 * Return :
 * Notes :
 */
void dveAVSimpleLoop(DveAVSession session);


/*
 * dveAVGetDveSession
 *   Returns the dveSession associated with the AV connection. Plugins may use
 *   this to possibly listen to other protocols, use the clipboard, etc.
 *
 * Input : session : The session returned by dveAVOpenSession()
 * Output :
 * Return : The dveSession associated with the AV connection.
 * Notes :
 */
DveSession dveAVGetDveSession(DveAVSession session);


/*
 * dveAVGetPort
 *   Returns the OpenDVE port number. Useful if dveAVOpenSession() was called
 *    with a port number argument of -1.
 *
 * Input : session : The session returned by dveAVOpenSession()
 * Output :
 * Return : The OpenDVE port number associated with the AV Connection.
 * Notes :
 */
DvePort dveAVGetPort(DveAVSession session);


/************** Calls to load and save user preferences ******************/

/* All the functions in this section are with reference to a particular
 * audio and video device. Once user preferences are saved, they are 
 * automatically loaded the next time the engine starts up. However, since 
 * an engine might continue to live beyond the lifespan of an application
 * it is probably better for an application to explicitly load user prefs
 * when it starts up. 
 */

/*
 * dveAVSavePrefs
 *    Save user prefs for a particular board id.
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio or dveAVDomainVideo
 *         boardid : Board id of audio/video device
 *
 * Output :
 * Return :
 * Notes :
 */
void dveAVSavePrefs(DveAVSession session, DveAVDomain domain, u_int32 boardid);


/*
 * dveAVLoadPrefs
 *    Load user prefs for a given board id
 * 
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio or dveAVDomainVideo
 *         boardid : Board id of audio/video device
 * 
 * Output :
 * Return :
 * Notes : An application must update its user-interface components
 *         after calling dveAVLoadPrefs() since values of board params 
 *         might have changed after the call. InSoft-provided GUIs will 
 *         automatically update themselves when dveAVLoadPrefs() is called.
 */
void dveAVLoadPrefs(DveAVSession session, DveAVDomain domain, u_int32 boardid);

/*
 * dveAVLoadDefaults
 *    Load default parameters for a given board id
 *
 * Input : session : The session returned by dveAVOpenSession()
 *         domain : dveAVDomainAudio or dveAVDomainVideo
 *         boardid : Board id of audio/video device
 *
 * Output :
 * Return :
 * Notes :
 */
void dveAVLoadDefaults(DveAVSession session, DveAVDomain domain,
		       u_int32 boardid);


/*********************** Miscellaneous functions **************************/


/*
 * dveAVStartDiags
 *   Start printing diagnostic messages on the console or to a log file. This
 *   is for future use.
 *
 * Input : 
 * Output :
 * Return :
 * Notes :
 */
void dveAVStartDiags(DveAVSession session, DveAVDomain domain);

/*
 * dveAVStopDiags
 *   Stop printing diagnostic messages. Currently not implemented.
 *
 * Input : 
 * Output :
 * Return :
 * Notes :
 */
void dveAVStopDiags(DveAVSession session, DveAVDomain domain);

#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_H_ */
