/* The audio domain */

/* General notes about the audio domain:
 * The audio domain is capable of supporting multiple audio devices ("boards")
 * on the same host, and multiple audio codecs ("compressions") per audio 
 * device. However, only one of these devices and codecs is allowed to be 
 * "active" at a time. When we send audio data, it is always with the 
 * currently active board and compression. Hence, in order to access a board, 
 * it is necessary to first make it "active". 
 *
 * A board is represented by means of an integer-valued "Board Id". A 
 * compression codec is represented by a four-character code. More 
 * than one board may support a given compression codec.
 *
 * Hence most audio parameters have a boardId parameter as part of their
 * specification : they don't make sense unless associated with a particular
 * board.
 */

/* Below is a list of audio parameter tags. These can be used with 
 * dveAV{Get/Set}Parameter. For all parameters, the type is given in
 * comments after the name. This is the type of the data associated with this
 * tag. Most parameters are read/write : you can do either a {Get/Set}Parameter
 * on them. Read-only parameters are explicitly mentioned. It is not an error
 * to attempt to set a read-only parameter: such an attempt is ignored, and
 * nothing is done. A '*' before the type indicates that the parameter in
 * question is integer-valued : you can use the convenience functions
 * dveAV{Set/Get}IntegerParameter to set and get their values. 
 */

#ifndef _DVE_AV_AUDIO_H_
#define _DVE_AV_AUDIO_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/AVCtl/AVCtl.h>
#include <ISaudio.h>

typedef enum {
  dveAVAudioTagInvalid=0,
  
  /* Type : DveAVAudioArgBoard; Currently active board */
  dveAVAudioTagCurrentBoard,         

  /* Type : DveAVAudioArgCurrentCompression; Currently active compression 
   * for a given board 
   */
  dveAVAudioTagCurrentCompression,   

  /* Type : DveAVAudioArgBoardInfo. Describes the available boards - this is
   * a read-only parameter
   */
  dveAVAudioTagBoardInfo,            

  /* Type : DveAVAudioArgInputPortInfo; Input port information for the
   * specified audio device. This is a read-only parameter.
   */
  dveAVAudioTagInputPortInfo, 

  /* Type : DveAVAudioArgOutputPortInfo; Output port information for the
   * specified audio device. This is a read-only parameter.
   */
  dveAVAudioTagOutputPortInfo, 


  /* The following apply to the currently active audio device */

  /* Type : DveAVAudioArgCompressionInfo; Describes the compressions 
   * available for this particular board. This parameter is read-only
   */
  dveAVAudioTagCompressionInfo,

  /* (*) Type : DveAVAudioArgBoardParam; The currently active input port 
   * (mic, line in)
   */
  dveAVAudioTagInputPort,   

  /* (*) Type : DveAVAudioArgBoardParam; The currently active output port 
   * (speaker, headphones, lineout)
   */
  dveAVAudioTagOutputPort,     

  /* (*) Type : DveAVAudioArgSilenceSensor; The current silence sensor value */
  dveAVAudioTagSilenceSensor,

  /* (*) Type : DveAVAudioArgEchoCancellation; The current echo-cancellation 
   * value 
   */
  dveAVAudioTagEchoCancellation,

  /* (*) Type : DveAVAudioArgAVSync; The current AV sync value */
  dveAVAudioTagAVSync,

  /* (*) Type : DveAVAudioArgBoardParam; Current play vol on a 0-100 scale */
  dveAVAudioTagPlayVolume,

  /* (*) Type : DveAVAudioArgBoardParam; Current mic vol on a 0-100 scale */
  dveAVAudioTagMicVolume,

  /* (*) Type : DveAVAudioArgSpeakerLevels; Current speaker levels */
  dveAVAudioTagSpeakerLevels, 

  /* Type : DveAVAudioArgAudioInfo; General audio board information */
  dveAVAudioTagAudioInfo

  } DveAVAudioTag;

/* Arg for dveAVAudioTagCurrentBoard */
typedef struct {
  u_int32 boardId;
  char name[64];
} DveAVAudioArgBoard,
  DveAVAudioArgCurrentBoard;

/* Arg for dveAVAudioTagBoardInfo */
typedef struct {
  int numBoards;
  DveAVAudioArgBoard *boards;
} DveAVAudioArgBoardInfo;

typedef struct {
  u_int32 boardId;
  char codec[5];  /* Four-character code for this compression codec; 
		   * NULL terminated
		   */
  char name[32];  /* The "official" name of this format, for possible display
		   * on the interface. This is a read-only parameter, and
		   * cannot be set.
		   */
} DveAVAudioArgCompression,
  DveAVAudioArgCurrentCompression;

/* Arg for dveAVVideoTagCompressionInfo */
typedef struct {
  u_int32 boardId;
  u_int32 numCompressions;

  DveAVAudioArgCompression *compressions;
} DveAVAudioArgCompressionInfo;


typedef struct {
  u_int32 boardId;
  Bool16 on, valid;   /* Set it on or off / Is it on or off? */
  u_int32 value;   
} DveAVAudioArgSilenceSensor,
  DveAVAudioArgEchoCancellation,
  DveAVAudioArgAVSync;

typedef struct {
  u_int32 boardId;
  u_int32 mask;
} DveAVAudioArgInputPortInfo,
  DveAVAudioArgOutputPortInfo;


typedef struct {
  u_int32 boardId;
  u_int8 *levels;
} DveAVAudioArgSpeakerLevels;

/* Most other parameters are just u_int32 's */
typedef struct {
  u_int32 boardId;
  Bool16 valid, align;   /* Is the value valid for this device? */
  int32 value;
} DveAVAudioArgBoardParam;

typedef struct {
  u_int32 boardId;
  char name[64];
  DveLocalAudioInputDeviceMask inputs;
  DveLocalAudioOutputDeviceMask outputs;
  u_int32 maxActive;
  DveAVAudioArgCompressionInfo cinfo;
} DveAVAudioBoard;

/* Type for dveAVAudioTagAudioInfo */
typedef struct {
  int numBoards;
  DveAVAudioBoard *boards;
} DveAVAudioArgAudioInfo;


#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_AUDIO_H_ */



