/* Various events in the audio, video and transport domain */

#ifndef _DVE_AV_EVENT_H_
#define _DVE_AV_EVENT_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <sys/types.h>
#include <IStypes.h>
#include <dve/AVCtl/core.h>

typedef enum {
  dveAVEventTagInvalid=0,
  dveAVEventTagEngineGone,           /* The engine died. The default action to
				      * this event is to quit. Unless a 
				      * callback has been supplied for this 
				      * event, the application will exit.
				      */

  dveAVEventTagParameterChanged,     /* An audio/video parameter reported by 
                                      * the device-specific capture process
				      * changed. The audio and video domains
				      * provide interval values (in Ms) at
				      * which to poll for changed parameters
				      */

  dveAVEventTagCaptureStopped,       /* Capture process died/stopped capture */
  dveAVEventTagTransferStopped,      /* A sending process to a member was
				      * terminated. This event is sent only if
				      * the sending process dies of an error
				      */
  dveAVEventTagStreamStarted,        /* Audio receiver/video viewer started */
  dveAVEventTagStreamStopped,        /* Audio/video viewer died */
  dveAVEventTagLocalMonitorStopped,  /* local monitor stopped */

  dveAVEventTagError,                /* An error of some sort occurred. This 
				      * includes situations such as the 
				      * capture process unable to open a 
				      * device, out of memory, etc
				      */
  dveAVEventTagStatus,               /* Status information */
  dveAVEventTagCapturePaused,        /* Audio/Video capture was paused */
  dveAVEventTagCaptureResumed,       /* Audio/Video capture was resumed */

  dveAVEventTagFilePlayStopped,      /* End of file playback */
  dveAVEventTagDTMFStopped,          /* A DTMF tone completed playback */

  /* The following event tags are used to pass events when the server is
   * doing negotiation on behalf of a client. They are not used otherwise.
   */
  dveAVEventTagMemberRejected,       /* A member rejected our offer */
  dveAVEventTagMemberAccepted,       /* A member accepted our offer */
  dveAVEventTagOfferAccepted,        /* We accepted an offer from a member */
  dveAVEventTagOfferRejected,        /* We rejected an offer from a member */

  /* The following event tags are only for plugins making H.320 calls */
  dveAVEventTagIncomingCall,         /* Incoming call */
  dveAVEventTagCallTerminated,       /* H320 call terminated */
  dveAVEventTagCapabilityReport      /* H320 capabilities from remote side */
  

} DveAVEventTag;

typedef struct {
  DveAVDomain domain;
  u_int32 tag, size;
  void *data;
} DveAVParameter;

typedef enum {
  dveAVParamChangeSourceInvalid = 0,
  dveAVParamChangeSourceThrottler,  /* Video bandwidth throttler */
  dveAVParamChangeSourceRemoteUser, /* Parameter was changed because of a
				     * request from a remote member. This could
				     * happen in a H.320 call, for example,
				     * if the remote member requested a mode
				     * change, or if some form of flow-control
				     * was in progress.
				     */
  dveAVParamChangeSourceLocalUser   /* Parameter was changed because either
				     * the local user changed it from the GUI,
				     * or was changed via the API
				     */
  } DveAVParamChangeSource;
  
typedef struct {
  int local;          /* If True, indicates local parameter change, ie. our
		       * audio/video parameter changed due to reason indicated
		       * by source. If False, indicates remote parameter
		       * change - some other member's remote parameter 
		       * changed. index indicates the identity of the member 
		       * whose parameter changed
		       */
  int index;          /* If remote, member index of member whose parameter
		       * changed 
		       */ 
  DveAVParamChangeSource source;    /* Who was responsible for changing 
				     * the param? 
				     */
  int numParams;
  DveAVParameter *params;
} DveAVEventArgParameterChanged;

typedef struct {
  int numCapabilities;
  DveAVParameter *capabilites;
} DveAVEventArgCapabilityReport;

typedef struct {
  int error;       /* If True, the tone was stopped because of an error; 
		    * otherwise, the tone completed playing normally 
		    */
} DveAVEventArgFilePlayStopped, DveAVEventArgDTMFStopped;

typedef struct {
  int memberIndex;  /* Index of Member from whom we were receiving the feed */

} DveAVEventArgStreamStopped,
  DveAVEventArgTransferStopped;

typedef struct {
  int memberIndex;    /* Index of Member from whom we are receiving the feed */
  int pid;              /* pid of viewer process for video only */
  char compression[4];  /* Type of compression being used */
  DveAVTransport transport; /* Actual transport information */

} DveAVEventArgStreamStarted;

typedef struct {
  u_int32 unsup;
} DveAVEventArgCaptureStopped, 
  DveAVEventArgLocalMonitorStopped, 
  DveAVEventArgError, 
  DveAVEventArgCapturePaused,
  DveAVEventArgCaptureResumed,
  DveAVEventArgEngineGone;

typedef struct {
  u_int32 level;     /* Status level */
  u_int32 code;      /* Status Code */
  char msg[256];     /* Status Message */

} DveAVEventArgStatus;

/* You get a memberRejected/accepted event if you are doing negotiation and 
 * a member accepts/rejects your offer. You also get it if you are in a 
 * H.320 call and the opposite party adds/drops audio or video.
 */
typedef struct {
  int memberIndex;      /* Index of remote member */
} DveAVEventArgMemberRejected, 
  DveAVEventArgMemberAccepted,
  DveAVEventArgOfferRejected,
  DveAVEventArgOfferAccepted;



typedef struct {
  int callId;        /* Call ID assigned to this call */
  DveAVTransport t;
  char nickName[32];
} DveAVEventArgIncomingCall;

typedef struct {
  int callId;
  u_int32 reason;
} DveAVEventArgCallTerminated;

  
typedef struct {
  u_int32 domain, tag;
  
  union {
    DveAVEventArgParameterChanged parameterChanged;
    DveAVEventArgStreamStarted streamStarted;
    DveAVEventArgStreamStopped streamStopped;
    DveAVEventArgCaptureStopped captureStopped;
    DveAVEventArgLocalMonitorStopped localMonitorStopped;
    DveAVEventArgError error;
    DveAVEventArgMemberRejected memberRejected;
    DveAVEventArgMemberAccepted memberAccepted;
    DveAVEventArgOfferRejected offerRejected;
    DveAVEventArgOfferAccepted offerAccepted;
    DveAVEventArgEngineGone engineGone;
    DveAVEventArgFilePlayStopped filePlayStopped;
    DveAVEventArgDTMFStopped dtmfStopped;
    DveAVEventArgIncomingCall incomingCall;
    DveAVEventArgCallTerminated callTerminated;
    DveAVEventArgCapabilityReport capabilityReport;
    DveAVEventArgStatus status;
  } ev;
} DveAVEvent;


typedef void (*DveAVEventCallback) (DveAVSession session,
				    int callbackId,
				    DveAVEvent *event,
				    void *userData);


#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_EVENT_H_ */
