/* The video domain */

/* General notes about the video domain:
 * The video domain is capable of supporting multiple video devices ("boards")
 * on the same host, and multiple video codecs ("compressions") per video 
 * device. However, only one of these devices and codecs is allowed to be 
 * "active" at a time. When we send video data, it is always with the 
 * currently active board and compression. Hence, in order to access a board, 
 * it is necessary to first make it "active". 
 *
 * A board is represented by means of an integer-valued "Board Id". A 
 * compression codec is represented by a four-character code (similar to that
 * used by Video For Windows to represent its video codecs). Note that more
 * than one board may support a given compression codec; For instance, most
 * InSoft-supported boards can compress CellB (ICLB). 
 *
 * Hence most video parameters have a boardId parameter as part of their
 * specification : they don't make sense unless associated with a particular
 * board.
 */

/* Below is a list of video parameter tags. These can be used with 
 * dveAV{Get/Set}Parameter. For all parameters, the type is given in
 * comments after the name. This is the type of the data associated with this
 * tag. Most parameters are read/write : you can do either a {Get/Set}Parameter
 * on them. Read-only parameters are explicitly mentioned. It is not an error
 * to attempt to set a read-only parameter: such an attempt is ignored, and
 * nothing is done. A '*' before the type indicates that the parameter in
 * question is integer-valued : you can use the convenience functions
 * dveAV{Set/Get}IntegerParameter to set and get their values.
 */

#ifndef _DVE_AV_VIDEO_H_
#define _DVE_AV_VIDEO_H_

#ifdef __cplusplus
extern "C" {
#endif

#include <dve/AVCtl/AVCtl.h>
#include <ISPlugins/ISVideoControlShm.h>

#define DVE_AV_VIDEO_NAMELEN     64
#define DVE_AV_VIDEO_BASE_PARAM  100
#define DVE_AV_VIDEO_MAX_PARAMS  DVE_maxParameters

typedef enum {
  dveAVVideoTagInvalid=0,

  /* (*) Type : DveAVVideoArgBoard; Currently active board */
  dveAVVideoTagCurrentBoard,         

  /* Type : DveAVVideoArgCurrentCompression; Currently active compression 
   * for a given board 
   */
  dveAVVideoTagCurrentCompression,   

  /* Type : DveAVVideoArgBoardInfo : Number of boards available, and their
   * names
   */
  dveAVVideoTagBoardInfo,            

  /* These tags all apply to the currently active board */

  /* Type : DveAVVideoArgCompressionInfo; Describes the compressions 
   * available for this particular board
   */
  dveAVVideoTagCompressionInfo,

  /* Type : DveAVVideoArgBandwidthInfo; Describes the bandwidths this board 
   * can handle 
   */
  dveAVVideoTagBandwidthInfo,

  /* Type : DveAVVideoArgFrameRateInfo : Describes the frame rates this
   * board can handle
   */
  dveAVVideoTagFrameRateInfo,

  /* Type : DveAVVideoArgFrameSizeInfo : Describes the frame sizes this
   * board can handle
   */
  dveAVVideoTagFrameSizeInfo,

  /* Type : DveAVVideoArgFavorInfo; Describes the parameters that are being
   * throttled on for this board
   */
  dveAVVideoTagFavorInfo,

  /* (*) Type : DveAVVideoArgNeedLocalMonitor; Boolean parameter that indicates
   * whether a separate "Local Monitor" is necessary for this board. For
   * most boards, this is True (> 0); But for boards such as Parallax, video 
   * capture is done by displaying overlay video on the screen and grabbing 
   * data off the overlaid image. For such boards, the capture process is the
   * local monitor, and the value is False (0).
   */
  dveAVVideoTagNeedLocalMonitor,

  /* (*) Type : DveAVVideoArgBoardParam; Current Frame rate */
  dveAVVideoTagFrameRate = DVE_FrameRate+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current bandwidth setting */
  dveAVVideoTagBandwidth = DVE_Bandwidth+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current quantization setting 
   * (for H261 and MPEG boards) 
   */
  dveAVVideoTagQuantization = DVE_Quantization+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current Brightness setting */
  dveAVVideoTagBrightness = DVE_Brightness+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current contrast setting */
  dveAVVideoTagContrast = DVE_Contrast+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current Hue setting */
  dveAVVideoTagHue = DVE_Hue+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current saturation setting */
  dveAVVideoTagSaturation = DVE_Saturation+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current Sharpness setting */
  dveAVVideoTagSharpness = DVE_Sharpness+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current Frame Size. 
   * 0 is the smallest size, 1 the next smallest, and so on.
   */
  dveAVVideoTagFrameSize = DVE_FrameSize+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current frame height. */
  dveAVVideoTagFrameHeight = DVE_FrameHeight+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current Frame width */
  dveAVVideoTagFrameWidth = DVE_FrameWidth+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current quality factor 
   * (usually used in JPEG boards) 
   */
  dveAVVideoTagQuality = DVE_Quality+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current input port 
   * (Analog, SVideo, etc).
   */
  dveAVVideoTagInput = DVE_Input+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Current format (NTSC, PAL, SECAM) */
  dveAVVideoTagFormat = DVE_Format+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Boolean; Set to true to ask the board 
   * to do gamma correction on its output.
   */
  dveAVVideoTagDoGamma = DVE_DoGamma+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Boolean; Set to true to tell the 
   * board to calibrate itself. Useful with self-calibrating cameras and 
   * boards.
   */ 
  dveAVVideoTagDoCalibration = DVE_DoCalibration+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Boolean; Set to True to tell the 
   * board to send out a refresh, key, or intra frame. Useful with boards 
   * that do inter-frame compression, such as H261 and CELLB boards.
   */
  dveAVVideoTagDoRefresh = DVE_DoRefresh+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Gives the device-id of the currently 
   * active device. This is a read-only parameter; It may not be set.
   */
  dveAVVideoTagDeviceId = DVE_HardwareDevice+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Value of the parameter currently 
   * being throttled on.
   */
  dveAVVideoTagFavor = DVE_Favor+DVE_AV_VIDEO_BASE_PARAM,

  /* (*) Type : DveAVVideoArgBoardParam; Sets/Gets the interval at which the
   * server looks for parameters reported by the device-specific capture
   * process
   */
  dveAVVideoTagPollInterval,

  /* Type : DveAVVideoArgVideoInfo. Get all available video information 
   * about all available boards.
   */
  dveAVVideoTagVideoInfo

} DveAVVideoTag;


/* Args for some of the tags */

/* Args for dveAVVideoTagCurrentBoard */
typedef struct {
  u_int32 boardId;   /* Board id for this board */
                     /* "Official" Board name to be represented in the 
		      * interface. This is a read-only parameter.
		      */
  char name[DVE_AV_VIDEO_NAMELEN];     

} DveAVVideoArgBoard,
  DveAVVideoArgCurrentBoard;


/* Arg for dveAVVideoTagBoardInfo */
typedef struct {
  u_int32 numBoards;
  
  DveAVVideoArgBoard *boards;
} DveAVVideoArgBoardInfo;

/* Arg for dveAVVideoTagCurrentCompression */
typedef struct {
  u_int32 boardId;
  char codec[5];  /* Four-character code for this compression codec; 
		   * NULL terminated
		   */
  char name[DVE_AV_VIDEO_NAMELEN];  
                  /* The "official" name of this format, for possible display
		   * on the interface. This is a read-only parameter, and
		   * cannot be set.
		   */
} DveAVVideoArgCompression;


/* Arg for dveAVVideoTagCompressionInfo */
typedef struct {
  u_int32 boardId;
  u_int32 numCompressions;

  DveAVVideoArgCompression *compressions;
} DveAVVideoArgCompressionInfo;

/* Arg(s) for dveAVVideoTagFrameRateInfo */
typedef struct {
  u_int32 boardId;
  u_int32 min, max;  /* Min and max values of the frame-rate */
} DveAVVideoArgFrameRateInfo;

/* Arg(s) for dveAVVideoTagFrameSizeInfo */
typedef enum {
  dveAVVideoFrameSizeMaskInvalid=0,
  dveAVVideoFrameSizeMaskSixteenth = 1,
  dveAVVideoFrameSizeMaskQuarter = 1 << 1,
  dveAVVideoFrameSizeMaskFull = 1 << 2
} DveAVVideoFrameSizeMask;

typedef struct {
  u_int32 boardId;
  u_int32 numSizes;   /* Number of sizes : 0 if any arbitrary size 
		       * is supported 
		       */
  u_int32 mask;       /* A collection of or'd DveAVVideoFrameSizeMask types */
} DveAVVideoArgFrameSizeInfo;

/* Arg(s) for dveAVVideoTagFavorInfo */
typedef struct {
  char name[DVE_AV_VIDEO_NAMELEN];     /* Name of throttle param */
  u_int32 id;                          /* id of this param */
} DveAVVideoArgThrottleParam;

typedef struct {
  u_int32 boardId;
  u_int32 numParams;   /* Number of throttle params */
  
  DveAVVideoArgThrottleParam *params;
} DveAVVideoArgFavorInfo;

/* Arg(s) for dveAVVideoTagBandwidthInfo */
typedef struct {
  u_int32 value;
  char string[DVE_AV_VIDEO_NAMELEN]; /* String to display in the interface */
} DveAVVideoArgBandwidth;

typedef struct {
  u_int32 boardId;
  u_int32 minBandwidth, maxBandwidth;  /* 0 if no limit */

  int numBandwidths;  /* Number of throttleable bandwidths */
  DveAVVideoArgBandwidth *bandwidths;
} DveAVVideoArgBandwidthInfo;

/* Arg(s) for dveAVVideoTagNeedLocalMonitor */
typedef struct {
  u_int32 boardId;
  Bool16 need, align;
} DveAVVideoArgNeedLocalMonitor;

/* Arg(s) for most other parameters */
typedef struct {
  u_int32 tag;
  u_int32 value;       
} DveAVVideoArgBoardParam;

/* Args for dveAVVideoTagVideoInfo */
typedef struct _DveAVVideoBoard DveAVVideoBoard;
struct _DveAVVideoBoard {
  int id;
  char *name;
  
  DveAVVideoArgCompressionInfo *compressionInfo;
  DveAVVideoArgFrameRateInfo *frameRateInfo;
  DveAVVideoArgFrameSizeInfo *frameSizeInfo;
  DveAVVideoArgBandwidthInfo *bandwidthInfo;
  DveAVVideoArgFavorInfo *favorInfo;

  DveAVVideoArgCompression *currentCompression;
  
  int numParams;
  DveAVVideoArgBoardParam *params;

};

typedef struct {
  int numBoards;
  DveAVVideoBoard **boards;
} DveAVVideoArgVideoInfo;

#ifdef __cplusplus
}
#endif

#endif /* _DVE_AV_VIDEO_H_ */

