#ifndef _PROFILE_H_
#define _PROFILE_H_

#ifdef __cplusplus
extern "C" {
#endif

/* Unless otherwise noted, the return value for a function returning
 *  int is 0 on success or <0 on error.  For a function returning a
 *  pointer type (e.g., pluginObject), a NULL return indicates an error.
 *  On error, consult dvePluginMgrErrno (see below) for the error.
 */

#include <sys/param.h>
#include <ISprofile.h>

#define         PLUGIN_UNMAPPED	0
#define         PLUGIN_MAPPED	1
#define         MAXPLUGINS      256

/*
 * Needed Types for Plugin Objects...
 */
typedef struct _pluginObjectType        *pluginObject;
 
struct _pluginObjectType {
        const char	*name;
        const char	*title;
        int         	processID;
        u_int32         pluginReady;
        DvePort         port;
        pluginObject    next;
        pluginObject    prev;
        u_int32         autoload;
        u_int32         createButton;
        u_int32         videoFrameCount;
        u_int32         averageBandwidth;
        u_int32         peakBandwidth;
	ISprofile	profile;
        char            basePath[MAXPATHLEN];
};

/* 
 * Callback invoked when the Plugin exits/Gets Ready... 
 */
typedef void (*PluginMgrCallbackProc)(pluginObject plug);

/*
 * Error conditions that could occur and are relayed back to the
 * user application via the dvePluginMgrErrno variable.
 */
typedef enum {
	dvePluginMgrUnknownError	= 0,
	dvePluginMgrLowMemory,
	dvePluginMgrInvalidPluginObject,
	dvePluginMgrInvalidSessionHandle,
	dvePluginMgrInvalidPluginPort,
	dvePluginMgrSendDataPacketError,
	dvePluginMgrInvalidInitFile,
	dvePluginMgrLibraryNotInitialized,
	dvePluginMgrErrorOnFork
} dvePluginMgrErrorTypes;

/* 
 * External variables for the user...
 */
extern u_int32		dvePluginMgrErrno;
extern pluginObject     dvePluginList[MAXPLUGINS];

/*
 * All API calls...
 *
 *****
 ***** Library initilization and cleanup routines
 *****
 */

/* 
 * Initializing the library
 *
 * This function initializes the Plugin Manager library and 
 * must be called before any other plugin management functions
 * can be used.
 *
 */

int
dvePluginMgrInitialize(DveSession dveSession, char *initFile,
		       PluginMgrCallbackProc readyCallback,
		       PluginMgrCallbackProc deathCallback);

/* 
 * Locating all available Plugins
 * 
 * After initializing the Plugin Manager library, this function
 * can be used to search all specified directories (from the .ini
 * file) and locate all available plugins.
 */
int
dvePluginMgrIdentifyAvailablePlugins(DveSession dveSession);


/* 
 * Close the Library
 * 
 * This function should be called at the end when the application is
 * ready to exit. This call enables the Plugin Manager to stop all
 * applications and close down cleanly.
 */
void
dvePluginMgrClose(void);


/*
 *****
 ***** Routines for starting and stopping Plugins.
 *****
 */

/*
 * Starting a Plugin
 * 
 * After the Plugin Manager library has been initialized and all available
 * plugins identified, this function can be used to start a plugin. A 
 * plugin can be started in either of two states: Mapped or Unmapped.
 */
int
dvePluginMgrStartPlugin(DveSession dveSession, pluginObject plug, 
						u_int32 mapState);
/*
 * Terminating a Plugin
 * 
 * This function can be used to send an OpenDVE message to a running
 * plugin to ask it to terminate itself.
 */ 
int
dvePluginMgrQuitPlugin(DveSession dveSession, pluginObject plug);

/*
 * Starting all Plugins
 * 
 * This is a convenience function to start all available plugins with
 * a single call.
 */
int
dvePluginMgrStartAllPlugins(DveSession dveSession, u_int32 mapState);

/*
 * Stopping all Plugins
 * 
 * This is a convenience function to terminate all available plugins with
 * a single call.
 */
int
dvePluginMgrStopAllPlugins(DveSession dveSession);


/*
 *****
 ***** Routines for controlling the mapped state of the Plugins that are
 ***** currently running.
 *****
 */

/*
 * Mapping a Plugin
 *
 * This function sends an OpenDVE message to a running plugin
 * to show (map) itself.
 *
 */
int
dvePluginMgrShowPlugin(DveSession dveSession, pluginObject plug);

/*
 * Mapping all Plugins
 *
 * This is a convenience function to show all running plugins with
 * a single call.
 */
int 
dvePluginMgrShowAllPlugins(DveSession dveSession);


/*
 * Hide a Plugin
 *
 * This function sends an OpenDVE message to a running plugin
 * to hide (unmap) itself.
 */

int
dvePluginMgrHidePlugin(DveSession dveSession, pluginObject plug);


/*
 * Hide all Plugins
 *
 * This is a convenience function to hide all running plugins with
 * a single call.
 *
 */
int 
dvePluginMgrHideAllPlugins(DveSession dveSession);


/*
 *****
 ***** General informational routines for the Plugins.
 *****
 */


/*
 *****
 ***** Routines for locating a Plugin object by different keys.
 *****
 */
/*
 * Locating a Plugin by Title
 * 
 * This function can be used to locate a plugin by its "title". If
 * successful, this function returns the associated pluginObject.
 */
pluginObject
dvePluginMgrLocatePluginByTitle(char *tag);

/*
 * Locating a Plugin by Name
 *
 * This function can be used to locate a plugin by its "name". If
 * successful, this function returns the associated pluginObject.
 */
pluginObject
dvePluginMgrLocatePluginByName(char *tag);

/*
 * Locating a Plugin by Process ID
 * 
 * This function can be used to locate a running plugin by its
 * process id. If successful, this function returns the associated
 * pluginObject.
 */
pluginObject
dvePluginMgrLocatePluginByProcessID(u_int32 pid);

/*
 * Locating a Plugin by Port Number
 * 
 * A plugin's port number is unique to it and is used when opening an
 * OpenDVE session. This function can be used to locate a plugin by
 * its port number. If successful, this function returns the associated
 * pluginObject.
 */
pluginObject
dvePluginMgrLocatePluginByPortNumber(DvePort port);

#ifdef __cplusplus
}
#endif

#endif
