/*
 *
 * Name:        Plugin_procs.c
 *
 * Description: These functions work hand-in-hand with those from
 *              OpenDVE_procs.c to provide OpenDVE messaging
 *              Tasks performed here are specific to Motif,
 *              while OpenDVE_procs.c contains generic functions.
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <Xm/Xm.h>
#include <stdio.h>
#include <sys/types.h>
#include <dve/dve.h>

#include <dve/Xm/recipientList.h>
#include "UxXt.h"

#include "SampleCameraPluginProtocol.h"
#include "Plugin_procs.h"

DveMotifRecipientList listB=NULL;
DvePartList partListB;

extern DveSession dveSession;
extern Widget topLevelShell1;
extern Widget scrolledList1;
extern Widget textField1;
extern Widget textField2;
extern Widget localscale;
extern Widget remotescale;
extern Widget footerlabel;

void introduceYourself(void);
void reportRemoteCameraPosition(DveUserHandle hPart, int pos);
void reportLocalCameraPosition(int pos);
void recipListCallback(DveMotifRecipientList list,
                       DveMotifRecipientListCallbackStruct* cbs);
void SendCurrentStatus(u_int32 messageType);
void starting(DveSession dveSession, DveEvent* event);

/*
 * Set the footer label used to hold the userName and hostName
 * strings in this example.
 */
void
mpSetFooter(const char *msg)
{
   XmString str=XmStringCreateSimple((char*)msg);
   XtVaSetValues(footerlabel,
                 XmNlabelString,str,
                 NULL);
   XmStringFree(str);
}

/*
 * topLevelShell1Init is the popupCallback function for our
 * topLevelShell1.  A DveSession is established.  The footer is 
 * set and the scale widgets are initialized.
 */
void topLevelShell1Init()
{
   DveConfParticipant LocalUser;
   char tmpString[DVE_MAXNAME+DVE_MAXHOST];

   if (dveGetLocalMember(dveSession, &LocalUser))
      fprintf(stderr, "I don't know who I am?\n");

   sprintf(tmpString, "%s@%s", LocalUser.userName, LocalUser.hostName);
   mpSetFooter(tmpString);

   XtVaSetValues(localscale,
                 XmNvalue, SCALE_POS_INITIAL,
                 NULL);
   XtVaSetValues(remotescale,
                 XmNvalue, SCALE_POS_INITIAL,
                 NULL);
   XmTextSetString(textField1, "0");
   XmTextSetString(textField2, "0");

}

/*
 * Exit has been selected from the File pull-down.
 */
void Exit_b_Callback() 
{
   HidePlugin();
}

/*
 * Received a dveEventTagMemberLeft.  The argument associated with
 * a dveEventTagMemberLeft is a DveConfParticipant.
 */
void
MemberLeft(DveConfParticipant member)
{
   fprintf(stderr, "User Left the Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

/*
 * Received a dveEventTagMemberAccepted.
 */
void
MemberAdded(DveConfParticipant member)
{
   fprintf(stderr, "User Added to Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

/*
 * Invoked from PluginMessageCallback() found in OpenDVE_procs.c.
 * Determine the type of message received and then update our
 * Plugin settings to match those of other conferenced members.
 */
void
ProcessPluginMessage(u_int32 messageType, void *data, DveUserHandle sender)
{
   int iPos;
   static DveCameraArg cArg;

   memcpy(&cArg, data, sizeof(DveCameraArg));

   switch(messageType)
   {
      case cameraTagLocal:

         /*
          * Position of the remote camera was changed.  Display changes
          * if its position is currently visible.
          */
      case cameraTagPositionReport:

         /*
          * Remote user has provided info on the remote camera position.
          */
      {
         DveUserHandle hPart = dveMotifRecipientListGetSelection(listB);
         if (hPart==sender)
         {
            char textString[5];

            int updated_pos=(int)ntohl(cArg.dwPosition);
            XtVaSetValues(remotescale,
                          XmNvalue, updated_pos,
                          NULL);
            sprintf(textString, "%d", updated_pos);
            XmTextSetString(textField2, (char *)textString);
         }
         break;
      }
      case cameraTagRemote:
      {

         /*
          * Remote user requested to change the position of the
          * local camera.                                     
          */
         char textString[5];
         static DveCameraArg data;

         int iPos = (int)ntohl(cArg.dwPosition);
         XtVaSetValues(localscale,
                       XmNvalue, iPos,
                       NULL);
         sprintf(textString, "%d", iPos);
         XmTextSetString(textField1, (char *)textString);

         reportLocalCameraPosition(iPos);
         break;
      }
      case cameraTagPositionRequest:

         /*
          * Remote user requested info on the local camera position.
          * Prepare argument and send the response.
          */
      {
         int value;
         static DveCameraArg data;
 
         value=(int)atoi((char *)XmTextGetString(textField1));
         data.dwPosition = (u_int32)value;
         htonDveCameraArg(&data);
         SendPluginMessageToMember(dveSession, REMOTE_CAMERA_PORT, sender, cameraTagPositionReport, &data, sizeof(data));
         break;
      }
      default:
         break;
   }
}

/*
 * When the starting event is received from the Conference Engine
 * the list is created using the OpenDVE toolkit function
 * dveMotifRecipientListCreate().
 */
void starting(DveSession dveSession, DveEvent* event)
{
   DvePartListClrAll(&partListB);
 
   listB=dveMotifRecipientListCreate(dveSession,
                                     scrolledList1,
                                     &partListB,
                                     True);
   if (!listB)
   {
      fprintf(stderr, "Unable to create DveMotifRecipientList.\n");
      return;
   }
 
   dveMotifRecipientListSetCallback(listB,recipListCallback);
}

/*
 * A conference member has left the session handle the Dve Event
 * and remove the member from listB. 
 */
void
MemberLeaving(DveSession dveSession, DveEvent* event)
{
   dveMotifRecipientListHandleDveEvent(listB, event);
   MemberLeft(event->arg.memberLeft.member);
}
 
/*
 * This is for when a conference member has accepted a conference
 * invitation.
 */
void
NewMemberAdded(DveSession dveSession, DveEvent* event)
{
   dveMotifRecipientListHandleDveEvent(listB, event);
   MemberAdded(event->arg.memberLeft.member);
}

/*
 * User activity has occurred on a DveMotifRecipientList.
 * Read the dveMotifRecipientListChangeTag from
 * DveMotifRecipientListCallbackStruct.
 */
void recipListCallback(DveMotifRecipientList list,
                       DveMotifRecipientListCallbackStruct* cbs)
{
   switch (cbs->tag)
   {
      case dveMotifRecipientListChangeTagChanged:
      {

         /*
          * Ask the remote camera about its position.  The scale
          * will be set when response is received.
          */
         SendPluginMessageToMember(dveSession, REMOTE_CAMERA_PORT, cbs->arg.changed.member, cameraTagPositionRequest, 0, 0);
         break;
      }
      default:
         return;
   }
}

/* 
 * Callback routine for the localscale widget.  When we've adjusted the 
 * localscale we must inform other conferenced members.
 */
void reportLocal(Widget UxWidget, XtPointer UxClientData, XtPointer UxCallbackArg)
{
   char textString[5];
   XmScaleCallbackStruct* cbs=(XmScaleCallbackStruct*) UxCallbackArg; 
   
   sprintf(textString, "%d", cbs->value);
   XmTextSetString(textField1, (char *)textString);
   reportLocalCameraPosition((int)cbs->value);
}

/* 
 * Local user is attempting to modify the position of a remote camera. 
 */
void reportRemote(Widget UxWidget, XtPointer UxClientData, XtPointer UxCallbackArg)
{
   char textString[5];
   DveUserHandle hPart;

   XmScaleCallbackStruct* cbs=(XmScaleCallbackStruct*) UxCallbackArg; 
   sprintf(textString, "%d", cbs->value);
   XmTextSetString(textField2, (char *)textString);
   
   hPart = dveMotifRecipientListGetSelection(listB);
   if (hPart != DVE_USER_HANDLE_INVALID)
   {
      reportRemoteCameraPosition((DveUserHandle)hPart, (int)cbs->value);
   }
}

/*
 * Adjusting the localscale widget from its associated textField.
 */
void textField1_Callback(Widget UxWidget, XtPointer UxClientData, XtPointer UxCallbackArg)
{
   int value;

   value=(int)atoi((char *)XmTextGetString(textField1));

   if ((value<SCALE_MIN) || (value>SCALE_MAX))
   {
      fprintf(stderr, "Invalid scale value\n");
      XmTextSetString(textField1, (char *)"");
   }
   else
   {
      XtVaSetValues(localscale,
                    XmNvalue, value,
                    NULL);
      reportLocalCameraPosition(value);
   }
}

/*
 * Adjusting the remotescale widget from its associated textField.
 */
void textField2_Callback(Widget UxWidget, XtPointer UxClientData, XtPointer UxCallbackArg)
{
   int value;

   value=(int)atoi((char *)XmTextGetString(textField2));

   if ((value<SCALE_MIN) || (value>SCALE_MAX))
   {
      fprintf(stderr, "Invalid scale value\n");
      XmTextSetString(textField2, (char *)"");
   }
   else
   {
      DveUserHandle hPart;

      hPart = dveMotifRecipientListGetSelection(listB);
      if (hPart != DVE_USER_HANDLE_INVALID)
      {
         XtVaSetValues(remotescale,
                       XmNvalue, value,
                       NULL);
         reportRemoteCameraPosition((DveUserHandle)hPart, (int)value);
      }
   }
}

/*
 * Inform others about our new local camera position.
 */
void reportLocalCameraPosition(int pos)
{
   static DveCameraArg data;

   data.dwPosition = (u_int32)pos;
   htonDveCameraArg(&data);
   SendPluginMessage(dveSession, REMOTE_CAMERA_PORT, cameraTagLocal, &data, sizeof(data));
}

/*
 * Inform hPart of our adjusted remote camera position.
 */
void reportRemoteCameraPosition(DveUserHandle hPart, int pos)
{
   static DveCameraArg data;

   data.dwPosition = (u_int32)pos;
   htonDveCameraArg(&data);
   SendPluginMessageToMember(dveSession, REMOTE_CAMERA_PORT, hPart, cameraTagRemote, &data, sizeof(data));
}

void 
HidePlugin()
{
   XtPopdown(topLevelShell1);
}

void 
ShowPlugin()
{
   XtPopup(topLevelShell1, XtGrabNone);
}

void
introduceYourself()
{
   /*
    * Initialize the connection with the conferenceEngine on the
    * REMOTE_CAMERA_PORT.
    */
   if (InitializeOpenDVE(UxAppContext, REMOTE_CAMERA_PORT)==NULL)
   {
      fprintf(stderr, "Initialization of OpenDVE Failed...exiting.\n\7");
      exit(0);
   }
}
