/*
 *
 * Name:        Plugin_procs.c
 *
 * Description: These functions work hand-in-hand with those from
 *              OpenDVE_procs.c to provide OpenDVE messaging
 *              Tasks performed here are specific to OpenLook,
 *              while OpenDVE_procs.c contains generic functions.
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <sys/param.h>
#include <sys/types.h>
#include <xview/xview.h>
#include <xview/panel.h>
#include <xview/textsw.h>
#include <xview/xv_xrect.h>
#include <group.h>
 
#include <dve/dve.h>
#include <dve/xview/recipientList.h>

#include "SampleCameraPluginProtocol.h"
#include "Plugin_procs.h"
#include "main_ui.h"

DveXViewRecipientList listB=NULL;
DvePartList partListB;

extern DveSession dveSession;
extern main_window1_objects    *Main_window1;

void reportRemoteCameraPosition(DveUserHandle hPart, int pos);
void reportLocalCameraPosition(int pos);
void recipListCallback(DveXViewRecipientList list,
                       DveXViewRecipientListCallbackStruct* cbs);
void ProcessPluginMessage(u_int32, void *, DveUserHandle);
void starting(DveSession, DveEvent*);
void MemberLeaving(DveSession, DveEvent*);
void NewMemberAdded(DveSession, DveEvent*);
void MemberLeft(DveConfParticipant);
void MemberAdded(DveConfParticipant member);

/*
 * Invoked from PluginMessageCallback() found in OpenDVE_procs.c.
 * Determine the type of message received and then update our
 * Plugin settings to match those of other conferenced members.
 */
void
ProcessPluginMessage(u_int32 messageType, void *data, DveUserHandle sender)
{
   static DveCameraArg cArg;

   memcpy(&cArg, data, sizeof(DveCameraArg));
 
   switch(messageType)
   {
      case cameraTagLocal:
 
         /*
          * Position of the remote camera was changed.  Display changes
          * if its position is currently visible.
          */
      case cameraTagPositionReport:
 
         /*
          * Remote user has provided info on the remote camera position.
          */
      {
	 DveUserHandle hPart = dveXViewRecipientListGetSelection(listB);
         if (hPart==sender)
         {
            int updated_pos=(int)ntohl(cArg.dwPosition);
            xv_set(Main_window1->slider2,
                   PANEL_VALUE, updated_pos,
                   NULL);
         }
         break;
      }
      case cameraTagRemote:
      {
 
         /*
          * Remote user requested to change the position of the
          * local camera.
          */
         char textString[5];
         static DveCameraArg data;
 
         int iPos = (int)ntohl(cArg.dwPosition);

         xv_set(Main_window1->slider1,
                PANEL_VALUE, iPos,
                NULL);
         reportLocalCameraPosition(iPos);
         break;
      }
      case cameraTagPositionRequest:
 
         /*
          * Remote user requested info on the local camera position.
          * Prepare argument and send the response.
          */
      {
         int value;
         static DveCameraArg data;
 
         value=(int)xv_get(Main_window1->slider1,
                           PANEL_VALUE, 
                           NULL);
         data.dwPosition = (u_int32)value;
         htonDveCameraArg(&data);
         SendPluginMessageToMember(dveSession, REMOTE_CAMERA_PORT, sender, cameraTagPositionReport, &data, sizeof(data));
         break;
      }
      default:
         ;
   }
}

/*
 * When the starting event is received from the Conference Engine
 * the list is created using the OpenDVE toolkit function
 * dveXViewRecipientListCreate().
 */
void starting(DveSession dveSession, DveEvent* event)
{
   DveUserHandle hPart;

   DvePartListClrAll(&partListB);
 
   listB=dveXViewRecipientListCreate(dveSession,
                                     (Xv_opaque)Main_window1->list1,
                                     &partListB,
                                     True);
   if (!listB)
   {
      fprintf(stderr, "Unable to create DveXViewRecipientList.\n");
      return;
   }

   dveXViewRecipientListSetCallback(listB, recipListCallback);
}

/*
 * A conference member has left the session handle the Dve Event
 * and remove the member from listB.
 */
void
MemberLeaving(DveSession dveSession, DveEvent* event)
{
   dveXViewRecipientListHandleDveEvent(listB, event);
   MemberLeft(event->arg.memberLeft.member);
}

/*
 * Received a dveEventTagMemberAccepted.
 */
void
MemberAdded(DveConfParticipant member)
{
   fprintf(stderr, "User Added to Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

/*
 * Received a dveEventTagMemberLeft.  The argument associated with
 * a dveEventTagMemberLeft is a DveConfParticipant.
 */
void
MemberLeft(DveConfParticipant member)
{
   fprintf(stderr, "User Left the Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

/*
 * User activity has occurred on a DveXViewRecipientList.
 * Read the dveXViewRecipientListChangeTag from
 * DveXViewRecipientListCallbackStruct.
 */
void recipListCallback(DveXViewRecipientList list,
                       DveXViewRecipientListCallbackStruct* cbs)
{
   switch (cbs->tag)
   {
      case dveXViewRecipientListChangeTagChanged:
      {
         /*
          * Ask the remote camera about its position.  The scale
          * will be set when response is received.
          */
         SendPluginMessageToMember(dveSession, REMOTE_CAMERA_PORT, cbs->arg.changed.member, cameraTagPositionRequest, 0, 0);
         break;
      }
      default:
         return;
   }
}

/*
 * This is for when a conference member has accepted a conference
 * invitation.
 */
void
NewMemberAdded(DveSession dveSession, DveEvent* event)
{
   dveXViewRecipientListHandleDveEvent(listB, event);
   MemberAdded(event->arg.memberLeft.member);
}

/*
 * Inform others about our new local camera position.
 */
void reportLocalCameraPosition(int pos)
{
   static DveCameraArg data;
 
   data.dwPosition = (u_int32)pos;
   htonDveCameraArg(&data);
   SendPluginMessage(dveSession, REMOTE_CAMERA_PORT, cameraTagLocal, &data, sizeof(data));
}
 
/*
 * Inform hPart of our adjusted remote camera position.
 */
void reportRemoteCameraPosition(DveUserHandle hPart, int pos)
{
   static DveCameraArg data;
 
   data.dwPosition = (u_int32)pos;
   htonDveCameraArg(&data);
   SendPluginMessageToMember(dveSession, REMOTE_CAMERA_PORT, hPart, cameraTagRemote, &data, sizeof(data));
}

/*
 * The Conference Manager has requested no display.
 */
void
HidePlugin()
{
   xv_set(Main_window1->window1, XV_SHOW, FALSE, NULL);
}
 
/*
 * The Conference Manager has requested a SHOW.
 */
void
ShowPlugin()
{
   xv_set(Main_window1->window1, XV_SHOW, TRUE, NULL);
}
