/*
 *
 * Name:        Plugin_procs.c
 *
 * Description: These functions work hand-in-hand with those from
 *              OpenDVE_procs.c to provide OpenDVE messaging
 *              and clipboarding.  Tasks performed here are specific
 *              to OL, while OpenDVE_procs.c contains generic
 *              functions.
 *
 * Change Log:
 *
 ********************************************************************
 *                                                                  *
 * (C) Copyright InSoft, Inc. 1995 All rights reserved              *
 *                                                                  *
 * You have a royalty-free right to use, modify, and reproduce the  *
 * the Sample Files (and/or any modified version) in any way you    *
 * find useful, provided that you agree that InSoft has no warranty *
 * obligations or liability for any Sample OpenDVE Plugin Files     *
 * which are modified.                                              *
 *                                                                  *
 ********************************************************************
 */

#include <stdio.h>
#include <sys/types.h>

#include <xview/xview.h>
#include <xview/panel.h>

#include <dve/dve.h>
#include <dve/clipboard.h>

#include "ISSamplePluginOL.h"
#include "SamplePluginProtocol.h"

extern DveSession dveSession;
extern DveClipboard clipboard;

/* 
 * Forward declarations.
 */
void InsertNewTextMessage(DveUserHandle sender, char *text, int size);
void SendCurrentStatus(u_int32 messageType);
void QueryUserForCurrentStatus(DveUserHandle member);

/*
 * Display the clipboard contents in the text subwindow.
 */
void	
ProcessClipboardData(char *format, char *data, int length)
{
   textsw_insert(Main_window1->textpane, "==========\n", 11); 
   textsw_insert(Main_window1->textpane, "Clipboard Contents:\n", 20);
   textsw_insert(Main_window1->textpane, data, length);
   textsw_insert(Main_window1->textpane, "\n", 1);
}

/*
 * XV_SHOW set to TRUE.
 */
void 
ShowPlugin(void)
{
   xv_set(Main_window1->window1, XV_SHOW, TRUE, NULL);
}

/*
 * XV_SHOW set to FALSE.
 */
void 
HidePlugin(void)
{
   xv_set(Main_window1->window1, XV_SHOW, FALSE, NULL);
}

/*
 * Call ShutdownOpenDVEClipboard to perform a dveClipClose().
 */
void 
CleanupPlugin(void)
{
   ShutdownOpenDVEClipboard(clipboard);
}

/*
 * We've just initiated the conference and we need to determine
 * what our current setting should be.  Use 
 * QueryUserForCurrentStatus() and our part->userHandle equals
 * the most senior conference member.
 */
void 
QueryPluginStatus(DveConfParticipant *part)
{
   fprintf(stderr, "We should ask Plugin status from %s@%s (%s)\n",
                    part->userName, part->hostName,
                    inet_ntoa(*(struct in_addr*)&((part->addr.v3))));
   QueryUserForCurrentStatus(part->userHandle);
}

/*
 * Received a dveEventTagMemberLeft.  The argument associated with
 * a dveEventTagMemberLeft is a DveConfParticipant.  Who left?
 */
void 
MemberLeft(DveConfParticipant member)
{
   fprintf(stderr, "User Left the Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

/*
 * Received a dveEventTagNewMember.  The argument associated with
 * a dveEventTagNewMember is a DveConfParticipant.  An invitation
 * has been sent to this user.  Please note, this member is
 * not active however until a MemberAccepted event is received.
 */
void 
MemberAdded(DveConfParticipant member)
{
   fprintf(stderr, "User Added to Conference - %s@%s (%s)\n",
                    member.userName, member.hostName,
                    inet_ntoa(*(struct in_addr*)&((member.addr.v3))));
}

/*
 * Invoked from PluginMessageCallback() found in OpenDVE_procs.c.
 * Determine the type of message received and then update our
 * Plugin settings to match those of other conferenced members.
 */
void 
ProcessPluginMessage(u_int32 messageType, void *data,
						DveUserHandle sender) 
{
   samplePluginMessageStructure message;
   char *tempPointer;

   memcpy(&message, data, sizeof(samplePluginMessageStructure));

   switch (messageType) {
      case TEMPERATURE_CHANGE:
         xv_set(Main_window1->tempslider, PANEL_VALUE, 
                                          ntohl(message.temperature), NULL);
         break;
      case MUSIC_CHANGE:
         xv_set(Main_window1->musicsetting, PANEL_VALUE, 
                                            ntohl(message.musicChoices), NULL);
         break;
      case DRINK_CHANGE:
         xv_set(Main_window1->drinksetting, PANEL_VALUE, 
                                            ntohl(message.drinkSelection), NULL);
         break;
      case TEXT_MESSAGE:
         tempPointer = ((samplePluginMessageStructure *)data)->message;
         InsertNewTextMessage(sender, tempPointer, ntohl(message.messageSize));
         break;
      case UPDATE_ALL_DATA:
         xv_set(Main_window1->tempslider, PANEL_VALUE, 
                                          ntohl(message.temperature), NULL);
         xv_set(Main_window1->musicsetting, PANEL_VALUE, 
                                            ntohl(message.musicChoices), NULL);
         xv_set(Main_window1->drinksetting, PANEL_VALUE, 
                                            ntohl(message.drinkSelection), NULL);
         break;
      case QUERY_PLUGIN_STATUS:
         textsw_insert(Main_window1->textpane, 
                       "Received a request for status...\n", 33);
         SendCurrentStatus(UPDATE_ALL_DATA);
         textsw_insert(Main_window1->textpane, 
                       "...sent current status of Plugin.\n", 34);
         break;
   }
}

/*
 * Notify other conference members of our current settings.  We have
 * either made a change or have received a request to provide status.
 */
void
SendCurrentStatus(u_int32 messageType)
{
   samplePluginMessageStructure *msg;

   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure));
   if (!msg)
      return;

   msg->protocolVersion = htonl(VERSION);

   /* 
    * Get the interface settings.
    */
   msg->temperature = htonl(xv_get(Main_window1->tempslider, PANEL_VALUE));
   msg->musicChoices = htonl(xv_get(Main_window1->musicsetting, PANEL_VALUE));
   msg->drinkSelection = htonl(xv_get(Main_window1->drinksetting, PANEL_VALUE));
   msg->messageSize = 0;

   (void) SendPluginMessage(dveSession, 
                            SAMPLE_PLUGIN_PORT, messageType, msg, 
                            sizeof(samplePluginMessageStructure));
   return;
}

/*
 * Query an individual conference participant for their current settings--
 * most likely the senior conference member.
 */
void 	
QueryUserForCurrentStatus(DveUserHandle member)
{
   samplePluginMessageStructure *msg;

   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure));
   if (!msg)
      return;

   msg->protocolVersion = VERSION;

   /* 
    * Get the interface settings...
    */
   msg->temperature = 0;
   msg->musicChoices = 0;
   msg->drinkSelection = 0;
   msg->messageSize = 0;

   SendPluginMessageToMember(dveSession, SAMPLE_PLUGIN_PORT,
                             member, QUERY_PLUGIN_STATUS, msg, 
                             sizeof(samplePluginMessageStructure));
   return;
}

/*
 * Send a text message to all conference participants.
 */
int
SendTextMessage(char *txtMessage, int length)
{
   samplePluginMessageStructure *msg;
   int rc;

   msg = (samplePluginMessageStructure *) dveMallocDataPacket(dveSession,
                                          sizeof(samplePluginMessageStructure) + length);
   if (!msg)
      return(-1);

   msg->protocolVersion = VERSION;
   msg->temperature = 0;
   msg->musicChoices = 0;
   msg->drinkSelection = 0;
   msg->messageSize = htonl(length);

   memcpy(msg->message, txtMessage, length);

   rc = SendPluginMessage(dveSession, 
                          SAMPLE_PLUGIN_PORT, TEXT_MESSAGE, msg, 
                          sizeof(samplePluginMessageStructure) + length);
		
   if (rc == 0)
      InsertNewTextMessage(0, txtMessage, length);
   return(rc);
}

/*
 * We have received a text message from a conference member.
 * Display this message in our plugin.
 */
void
InsertNewTextMessage(DveUserHandle sender, char *text, int size)
{
   DveConfParticipant member;
   char newMessage[1024];
   char fullName[256];

   textsw_insert(Main_window1->textpane, "==========\n", 11); 

   if (sender == 0)
   {
      sprintf(newMessage, "Message from: ME!");
   }
   else if (dveGetOneMember(dveSession, sender, &member) < 0)
   {
      sprintf(newMessage, "Message From: Unknown User");
   }
   else
   {
      sprintf(newMessage, "Message From: %s@%s", member.userName,
                                                 member.hostName);
   }

   textsw_insert(Main_window1->textpane, newMessage, strlen(newMessage));
   textsw_insert(Main_window1->textpane, "\n", 1);
   textsw_insert(Main_window1->textpane, text, size);
   textsw_insert(Main_window1->textpane, "\n", 1);
}
