/*
 * $RCSfile$
 * $Date$
 * $Revision$
 * $Locker$
 * 
 * Description:
 *  Low-level definitions for OpenDVE, needed by both the library and the
 *   engine.
 *
 * Requires:
 *  <netinet/in.h>
 *  <IStypes.h>
 *
 * Includes:
 *  <string.h> (in C++ only)
 *
 * Implemented by:
 *  dve.c
 *  pluginmgr.c (?)
 *
 * Types:
 *  DvePartList
 *  DveProtocol
 *  DvePort
 *  DveIPA
 *  DveUserHandle
 *  DveConfParticipant
 *  ConfdState
 *  DveVersion
 *  DveRoutingFlag
 *  DveDataPacket
 *
 * Functions:
 *  DvePartListSet()
 *  DvePartListClr()
 *  DvePartListIsSet()
 *  DvePartListClrAll()
 *  DvePartListSetAll()
 *
 * Macros:
 *  DVE_MAXNAME
 *  DVE_MAXHOST
 *  DVE_MAXFULLNAME
 *  DVE_MAXREALNAME
 *
 * Author:
 *  francis, ?? Feb 1994
 *
 * Change Log:
 *  $Log$
 */
#ifndef _DVE_LOW_PROTOCOL_H_
#define _DVE_LOW_PROTOCOL_H_

#ifdef __cplusplus
extern "C" {
#endif

#define IS_IDENT_STRING "@(#) $Id$"
#include <ISident.h>

/* Used to pad after a type that may not be the same size on all
 * platforms.  DO NOT use in any message that has to be sent over the
 * net; use only for local messages.
 */
#define ALIGNSIZE(type) (4-(sizeof(type)%4))

/* Max # of chars in username, hostname, full address, personal name */
#define DVE_MAXNAME         32
#define DVE_MAXHOST         64
#define DVE_MAXFULLNAME     (DVE_MAXNAME+DVE_MAXHOST)
#define DVE_MAXREALNAME     64

/* Max # of members of conference */
#define DVE_MAX_PARTICIPANTS 24
#define DVE_MAX_USERHANDLE 255
/* # of bytes needed to hold DVE_MAX_PARTICIPANTS flags in DvePartList */
#define DVE_PARTLIST_BYTES ((DVE_MAX_USERHANDLE+7)/8)

/* The following imitates the fd_mask stuff of select() */

typedef	struct {
  char bits[DVE_PARTLIST_BYTES];
  char align[4-(DVE_PARTLIST_BYTES%4)];
} DvePartList;

#ifdef __cplusplus

#include <string.h> /*for memset()*/

/* Set the flag for participant # partNum (this is a DveUserHandle)
 * in *partList */
inline void DvePartListSet(int partNum, DvePartList* partList)
{
  partList->bits[partNum/8] |= ('\01' << (partNum % 8));
}

/* Clear the flag for participant # partNum in *partList */
inline void DvePartListClr(int partNum, DvePartList* partList)
{
  partList->bits[partNum/8] &= ~('\01' << (partNum % 8));
}

/* Test the flag for participant # partNum in *partList */
inline Bool16 DvePartListIsSet(int partNum, DvePartList* partList)
{
  return (partList->bits[partNum/8] & ('\01' << (partNum % 8)));
}

/* Set *partList to empty */
inline void DvePartListClrAll(DvePartList* partList)
{
  memset((char *)(partList),0,sizeof(DvePartList));
}

/* Set *partList to contain everyone in the conference */
#if 0
inline void DvePartListSetAll(DvePartList* partList)
{
  memset((char *)(partList),255,sizeof(DvePartList));
}
#else
void DvePartListSetAll(DvePartList* partList);
Bool16 DvePartListNonEmpty(DvePartList* partList);
#endif

#else /*!__cplusplus*/

/* See above */
void DvePartListSet(int partNum, DvePartList* partList);
void DvePartListClr(int partNum, DvePartList* partList);
Bool16 DvePartListIsSet(int partNum, DvePartList* partList);
void DvePartListClrAll(DvePartList* partList);
void DvePartListSetAll(DvePartList* partList);
Bool16 DvePartListNonEmpty(DvePartList* partList);

#endif /*__cplusplus*/

/* Plugin port specifier */
typedef u_int32 DvePort;

/* Internet protocol addr. (in network order) */
typedef struct {
  struct in_addr v3; /* 4-byte IPv3 address */
  char reserved[16]; /* Reserved for IPv4, which will have larger addresses.
          * 16 may be excessive, but we need to be sure. */
} DveIPA;

/* Handle (ID#) of user within the conference. */
typedef u_int8 DveUserHandle;

/* Invalid user handle (i.e., not yet assigned, not found,...) */
#define DVE_USER_HANDLE_INVALID 0
/* User handle of initiator */
#define DVE_USER_HANDLE_INITIATOR 1

/* Structure giving info on a participant */
typedef struct{
  DveIPA addr;
  char userName[DVE_MAXNAME];
  char hostName[DVE_MAXHOST];
  DveUserHandle userHandle;
  char align[3];
} DveConfParticipant;

#define DVE_MAX_INTERFACES 16

typedef struct {
  struct {
    DveIPA host;
    u_int16 port,reserved[3]; /* port # in network order */
  } addrs[DVE_MAX_INTERFACES];
  u_int32 numAddrs;
} DveIPAList;

typedef struct {
  DveIPAList addrs;
  DveConfParticipant part;
} DveConfParticipantLong;

typedef struct {
  u_int8 numMembers,recipientIndex,align[2];
  struct {
    DveConfParticipantLong part;
    Bool16 active;
    u_int16 align;
  } members[DVE_MAX_PARTICIPANTS];
} ConfdMemberList;

/* State for the conference engine */
typedef enum {
  confdStateTagInvalid=0,
  confdStateTagDead,
  confdStateTagRunning,
  confdStateTagPrompting,
  confdStateTagAccepted
  } ConfdStateTag;

typedef enum {
  confdRejectionReasonInvalid=0,
  confdRejectionReasonNotPresent,
  confdRejectionReasonLineBusy,
  confdRejectionReasonRejected,
  confdRejectionReasonCantConnect_HostUnknown,
  confdRejectionReasonCantConnect_HostUnreachable,
  confdRejectionReasonCantConnect_NetworkUnreachable,
  confdRejectionReasonCantConnect_NoRegistrar,
  confdRejectionReasonCantConnect_NoEngine,
  confdRejectionReasonCantConnect_Local,
  confdRejectionReasonCantConnect_Unknown
  } ConfdRejectionReason;

/* Version # */
typedef struct {
  u_int16 major,minor;
} DveVersion;

/* Flags specifying how a packet should be routed */
typedef enum
{
  dveRoutingFlagNull=0,
  dveRoutingFlagGiveBounce=1, /* If packet can't be delivered, bounce */
  dveRoutingFlagPassiveOK=2   /* OK to send to passive members */
  } DveRoutingFlag;

typedef struct {
  u_int32 size;
  DveVersion protVersion;
  DvePort destPort, srcPort;
  u_int32 routingFlags;
  u_int32 pluginProtocol; /* Which plugin's protocol is being used?
			   * Same as plugin's port # */
  u_int32 tag; /* Message tag, for private use by the plugin. */
  DvePartList destParts; /* Participants to receive the message. */
  char data[1]; /* Actually data[dataLen],
		 * where dataLen=size-sizeof(DveDataPacket)+1,
		 * but try telling that to a compiler. :-) */
} DveDataPacket;

/* Note that, if the destination plugin port is the same as the source
 * plugin port, the sender does not receive its own message.
 */

typedef enum {
  dveRejectionTagInvalid=0,
  dveRejectionTagBusy,
  dveRejectionTagDeliberate,
  dveRejectionTagOutOfOffice
  } DveRejectionTag;

#ifdef __cplusplus
}
#endif

#endif /*_DVE_LOW_PROTOCOL_H_*/
